@@ -5,7 +5,7 @@ package cloudi
 //
 // MIT License
 //
-// Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -141,7 +141,7 @@ func API(threadIndex uint32, state interface{}) (*Instance, error) {
 	callbacks := make(map[string]*list.List)
 	bufferRecv := new(bytes.Buffer)
 	bufferRecv.Grow(int(bufferSize))
-	timeoutTerminate := uint32(1000)
+	timeoutTerminate := uint32(10) // TIMEOUT_TERMINATE_MIN
 	api := &Instance{state: state, socket: socket, useHeader: useHeader, fragmentSize: bufferSize, fragmentRecv: fragmentRecv, callbacks: callbacks, bufferRecv: bufferRecv, timeoutTerminate: timeoutTerminate}
 	var init []byte
 	init, err = erlang.TermToBinary(erlang.OtpErlangAtom("init"), -1)
@@ -716,19 +716,20 @@ func (api *Instance) callback(command uint32, name, pattern string, requestInfo,
 		responseInfo, response, err := api.callbackExecute(function, command, name, pattern, requestInfo, request, timeout, priority, transId, pid)
 		if err != nil {
 			switch err.(type) {
-			case *InvalidInputError:
-				return err
 			case *MessageDecodingError:
-				return err
+				api.terminate = true
+				err = nil
 			case *TerminateError:
-				return err
+				err = nil
 			case *ReturnAsyncError:
 				return nil
 			case *ReturnSyncError:
+				api.terminate = true
 				return err
 			case *ForwardAsyncError:
 				return nil
 			case *ForwardSyncError:
+				api.terminate = true
 				return err
 			default:
 				os.Stderr.WriteString(err.Error() + "\n")
@@ -740,17 +741,18 @@ func (api *Instance) callback(command uint32, name, pattern string, requestInfo,
 		responseInfo, response, err := api.callbackExecute(function, command, name, pattern, requestInfo, request, timeout, priority, transId, pid)
 		if err != nil {
 			switch err.(type) {
-			case *InvalidInputError:
-				return err
 			case *MessageDecodingError:
-				return err
+				api.terminate = true
+				err = nil
 			case *TerminateError:
-				return err
+				err = nil
 			case *ReturnAsyncError:
+				api.terminate = true
 				return err
 			case *ReturnSyncError:
 				return nil
 			case *ForwardAsyncError:
+				api.terminate = true
 				return err
 			case *ForwardSyncError:
 				return nil
@@ -857,7 +859,10 @@ func (api *Instance) handleEvents(external bool, reader *bytes.Reader, command u
 func (api *Instance) pollRequest(timeout int32, external bool) (bool, error) {
 	var err error
 	if api.terminate {
-		return false, nil
+		if external {
+			return false, nil
+		}
+		return true, terminateErrorNew(api.timeoutTerminate)
 	} else if external && !api.initializationComplete {
 		var polling []byte
 		polling, err = erlang.TermToBinary(erlang.OtpErlangAtom("polling"), -1)
@@ -1063,6 +1068,9 @@ func (api *Instance) pollRequest(timeout int32, external bool) (bool, error) {
 			if err != nil {
 				return false, err
 			}
+			if api.terminate {
+				return false, nil
+			}
 		case messageRecvAsync:
 			fallthrough
 		case messageReturnSync:
@@ -1239,25 +1247,76 @@ func (api *Instance) Shutdown(extra ...interface{}) error {
 	return nil
 }
 
-func (api *Instance) textKeyValueParse(text []byte) map[string][]string {
-	result := map[string][]string{}
-	data := bytes.Split(text, []byte{0})
-	for i := 0; i < len(data)-1; i += 2 {
-		key := string(data[i])
-		value := string(data[i+1])
-		current := result[key]
+func textPairsParse(text []byte) map[string][]string {
+	pairs := map[string][]string{}
+	textSegments := bytes.Split(text, []byte{0})
+	for i := 0; i < len(textSegments)-1; i += 2 {
+		key := string(textSegments[i])
+		value := string(textSegments[i+1])
+		current := pairs[key]
 		if current == nil {
-			result[key] = []string{value}
+			pairs[key] = []string{value}
 		} else {
-			result[key] = append(current, value)
+			pairs[key] = append(current, value)
 		}
 	}
-	return result
+	return pairs
 }
 
-// InfoKeyValueParse parses "text_pairs" in service request info
-func (api *Instance) InfoKeyValueParse(messageInfo []byte) map[string][]string {
-	return api.textKeyValueParse(messageInfo)
+func textPairsNew(pairs map[string][]string, response bool) ([]byte, error) {
+	var textBuffer = new(bytes.Buffer)
+	var err error
+	for key, values := range pairs {
+		for i := 0; i < len(values); i++ {
+			_, err = textBuffer.WriteString(key)
+			if err != nil {
+				return nil, err
+			}
+			err = textBuffer.WriteByte(0)
+			if err != nil {
+				return nil, err
+			}
+			_, err = textBuffer.WriteString(values[i])
+			if err != nil {
+				return nil, err
+			}
+			err = textBuffer.WriteByte(0)
+			if err != nil {
+				return nil, err
+			}
+		}
+	}
+	if response && textBuffer.Len() == 0 {
+		err = textBuffer.WriteByte(0)
+		if err != nil {
+			return nil, err
+		}
+	}
+	text := textBuffer.Bytes()
+	return text, nil
+}
+
+// InfoKeyValueParse decodes service request info key/value data
+func InfoKeyValueParse(info []byte) map[string][]string {
+	return textPairsParse(info)
+}
+
+// InfoKeyValueNew encodes service response info key/value data
+func InfoKeyValueNew(pairs map[string][]string, extra ...interface{}) ([]byte, error) {
+	extraArity := len(extra)
+	if extraArity > 1 {
+		return nil, invalidInputErrorNew()
+	}
+	response := true
+	for _, extraArg := range extra {
+		switch arg := extraArg.(type) {
+		case bool:
+			response = arg
+		default:
+			return nil, invalidInputErrorNew()
+		}
+	}
+	return textPairsNew(pairs, response)
 }
 
 func (api *Instance) send(data []byte) error {
