@@ -3,44 +3,34 @@
 
 {-
 
-  BSD LICENSE
+  MIT License
 
-  Copyright (c) 2017, Michael Truog <mjtruog at gmail dot com>
-  All rights reserved.
+  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are met:
+  Permission is hereby granted, free of charge, to any person obtaining a
+  copy of this software and associated documentation files (the "Software"),
+  to deal in the Software without restriction, including without limitation
+  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+  and/or sell copies of the Software, and to permit persons to whom the
+  Software is furnished to do so, subject to the following conditions:
 
-      * Redistributions of source code must retain the above copyright
-        notice, this list of conditions and the following disclaimer.
-      * Redistributions in binary form must reproduce the above copyright
-        notice, this list of conditions and the following disclaimer in
-        the documentation and/or other materials provided with the
-        distribution.
-      * All advertising materials mentioning features or use of this
-        software must display the following acknowledgment:
-          This product includes software developed by Michael Truog
-      * The name of the author may not be used to endorse or promote
-        products derived from this software without specific prior
-        written permission
+  The above copyright notice and this permission notice shall be included in
+  all copies or substantial portions of the Software.
 
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-  DAMAGE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+  DEALINGS IN THE SOFTWARE.
 
  -}
 
+-- | Haskell <https://cloudi.org/api.html#1_Intro CloudI API>.
+-- Example usage is available in the
+-- <https://cloudi.org/tutorials.html#cloudi_examples integration tests>.
+
 module Foreign.CloudI
     ( Instance.RequestType(..)
     , Instance.Source
@@ -52,6 +42,7 @@ module Foreign.CloudI
     , messageDecodingError
     , terminateError
     , Exception
+    , Result
     , api
     , threadCount
     , subscribe
@@ -77,12 +68,14 @@ module Foreign.CloudI
     , timeoutSync
     , timeoutTerminate
     , poll
+    , shutdown
     , threadCreate
     , threadsWait
     , infoKeyValueParse
+    , infoKeyValueNew
     ) where
 
-import Prelude hiding (init,length)
+import Prelude hiding (init,length,(<>))
 import Data.Bits (shiftL,(.|.))
 import Data.Maybe (fromMaybe)
 import Data.Typeable (Typeable)
@@ -105,6 +98,7 @@ import qualified Foreign.C.Types as C
 import qualified Foreign.Erlang as Erlang
 import qualified Foreign.CloudI.Instance as Instance
 import qualified System.IO as SysIO
+import qualified System.IO.Error as SysIOErr
 import qualified System.IO.Unsafe as Unsafe
 import qualified System.Posix.Env as POSIX (getEnv)
 type Array = IArray.Array
@@ -149,6 +143,8 @@ data Message =
         Int, Int, ByteString, Source)
     | MessageKeepalive
 
+-- | a null trans_id is used to check for a timeout or
+-- to get the oldest response with recv_async
 transIdNull :: ByteString
 transIdNull = Char8.pack $ List.replicate 16 '\0'
 
@@ -164,6 +160,7 @@ data Exception s =
     | ReturnAsync (Instance.T s)
     | ForwardSync (Instance.T s)
     | ForwardAsync (Instance.T s)
+    | Terminate (Instance.T s)
     deriving (Show, Typeable)
 
 instance Typeable s => Exception.Exception (Exception s)
@@ -184,9 +181,14 @@ data CallbackResult s =
 
 type Result a = Either String a
 
-api :: Typeable s => Int -> s ->
+infixr 4 <>
+(<>) :: Monoid.Monoid m => m -> m -> m
+(<>) = Monoid.mappend
+
+-- | creates an instance of the CloudI API
+api :: Typeable s => Int -> s -> Maybe Bool ->
     IO (Result (Instance.T s))
-api threadIndex state = do
+api threadIndex state terminateReturnValueOpt = do
     SysIO.hSetEncoding SysIO.stdout SysIO.utf8
     SysIO.hSetBuffering SysIO.stdout SysIO.LineBuffering
     SysIO.hSetEncoding SysIO.stderr SysIO.utf8
@@ -195,28 +197,32 @@ api threadIndex state = do
     bufferSizeValue <- POSIX.getEnv "CLOUDI_API_INIT_BUFFER_SIZE"
     case (protocolValue, bufferSizeValue) of
         (Just protocol, Just bufferSizeStr) ->
-            let bufferSize = read bufferSizeStr :: Int
+            let terminateReturnValue = fromMaybe True terminateReturnValueOpt
+                terminateException = not terminateReturnValue
+                bufferSize = read bufferSizeStr :: Int
                 fd = C.CInt $ fromIntegral (threadIndex + 3)
                 useHeader = protocol /= "udp"
-                timeoutTerminate' = 1000
+                timeoutTerminate' = 10 -- TIMEOUT_TERMINATE_MIN
                 initTerms = Erlang.OtpErlangAtom (Char8.pack "init")
             in
             case Erlang.termToBinary initTerms (-1) of
                 Left err ->
                     return $ Left $ show err
                 Right initBinary -> do
-                    api0 <- Instance.make state protocol fd
-                        useHeader bufferSize timeoutTerminate'
+                    api0 <- Instance.make state terminateException
+                        protocol fd useHeader bufferSize timeoutTerminate'
                     send api0 initBinary
                     result <- pollRequest api0 (-1) False
                     case result of
                         Left err ->
+                            -- Terminate exception not used here
                             return $ Left err
                         Right (_, api1) ->
                             return $ Right api1
         (_, _) ->
             return $ Left invalidInputError
 
+-- | returns the thread count from the service configuration
 threadCount :: IO (Result Int)
 threadCount = do
     threadCountValue <- POSIX.getEnv "CLOUDI_API_INIT_THREAD_COUNT"
@@ -226,6 +232,7 @@ threadCount = do
         Just threadCountStr ->
             return $ Right (read threadCountStr :: Int)
 
+-- | subscribes to a service name pattern with a callback
 subscribe :: Instance.T s -> ByteString -> Instance.Callback s ->
     IO (Result (Instance.T s))
 subscribe api0 pattern f =
@@ -240,9 +247,12 @@ subscribe api0 pattern f =
             send api0 subscribeBinary
             return $ Right $ Instance.callbacksAdd api0 pattern f
 
+-- | returns the number of subscriptions for a single service name pattern
 subscribeCount :: Typeable s => Instance.T s -> ByteString ->
     IO (Result (Int, Instance.T s))
-subscribeCount api0 pattern =
+subscribeCount api0@Instance.T{
+      Instance.terminateException = terminateException}
+    pattern =
     let subscribeCountTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "subscribe_count")
             , Erlang.OtpErlangString pattern]
@@ -255,10 +265,14 @@ subscribeCount api0 pattern =
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{Instance.subscribeCount = count}) ->
                     return $ Right (count, api1)
 
+-- | unsubscribes from a service name pattern once
 unsubscribe :: Instance.T s -> ByteString ->
     IO (Result (Instance.T s))
 unsubscribe api0 pattern =
@@ -273,11 +287,13 @@ unsubscribe api0 pattern =
             send api0 unsubscribeBinary
             return $ Right $ Instance.callbacksRemove api0 pattern
 
+-- | sends an asynchronous service request
 sendAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     Maybe Int -> Maybe ByteString -> Maybe Int ->
     IO (Result (ByteString, Instance.T s))
 sendAsync api0@Instance.T{
-      Instance.timeoutAsync = timeoutAsync'
+      Instance.terminateException = terminateException
+    , Instance.timeoutAsync = timeoutAsync'
     , Instance.priorityDefault = priorityDefault}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutAsync' timeoutOpt
@@ -299,15 +315,20 @@ sendAsync api0@Instance.T{
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{Instance.transId = transId}) ->
                     return $ Right (transId, api1)
 
+-- | sends a synchronous service request
 sendSync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     Maybe Int -> Maybe ByteString -> Maybe Int ->
     IO (Result (ByteString, ByteString, ByteString, Instance.T s))
 sendSync api0@Instance.T{
-      Instance.timeoutSync = timeoutSync'
+      Instance.terminateException = terminateException
+    , Instance.timeoutSync = timeoutSync'
     , Instance.priorityDefault = priorityDefault}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutSync' timeoutOpt
@@ -329,18 +350,24 @@ sendSync api0@Instance.T{
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{
                       Instance.responseInfo = responseInfo
                     , Instance.response = response
                     , Instance.transId = transId}) ->
                     return $ Right (responseInfo, response, transId, api1)
 
+-- | sends asynchronous service requests to all subscribers
+-- of the matching service name pattern
 mcastAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     Maybe Int -> Maybe ByteString -> Maybe Int ->
     IO (Result (Array Int ByteString, Instance.T s))
 mcastAsync api0@Instance.T{
-      Instance.timeoutAsync = timeoutAsync'
+      Instance.terminateException = terminateException
+    , Instance.timeoutAsync = timeoutAsync'
     , Instance.priorityDefault = priorityDefault}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutAsync' timeoutOpt
@@ -362,10 +389,14 @@ mcastAsync api0@Instance.T{
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{Instance.transIds = transIds}) ->
                     return $ Right (transIds, api1)
 
+-- | forwards a service request to a different service name
 forward_ :: Typeable s =>
     Instance.T s -> Instance.RequestType -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
@@ -376,21 +407,13 @@ forward_ api0 Instance.SYNC = forwardSync api0
 forwardAsyncI :: Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-forwardAsyncI api0@Instance.T{
-      Instance.requestTimeoutAdjustment = requestTimeoutAdjustment
-    , Instance.requestTimer = requestTimer
-    , Instance.requestTimeout = requestTimeout}
-    name responseInfo response timeout priority transId pid = do
-    timeoutNew <- if requestTimeoutAdjustment && timeout == requestTimeout then
-            timeoutAdjustment requestTimer timeout
-        else
-            return timeout
+forwardAsyncI api0 name responseInfo response timeout priority transId pid = do
     let forwardTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "forward_async")
             , Erlang.OtpErlangString name
             , Erlang.OtpErlangBinary responseInfo
             , Erlang.OtpErlangBinary response
-            , Erlang.OtpErlangInteger timeoutNew
+            , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangInteger priority
             , Erlang.OtpErlangBinary transId
             , Erlang.OtpErlangPid pid]
@@ -401,6 +424,7 @@ forwardAsyncI api0@Instance.T{
             send api0 forwardBinary
             return $ Right api0
 
+-- | forwards an asynchronous service request to a different service name
 forwardAsync :: Typeable s => Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO ()
@@ -416,21 +440,13 @@ forwardAsync api0 name responseInfo response timeout priority transId pid = do
 forwardSyncI :: Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-forwardSyncI api0@Instance.T{
-      Instance.requestTimeoutAdjustment = requestTimeoutAdjustment
-    , Instance.requestTimer = requestTimer
-    , Instance.requestTimeout = requestTimeout}
-    name responseInfo response timeout priority transId pid = do
-    timeoutNew <- if requestTimeoutAdjustment && timeout == requestTimeout then
-            timeoutAdjustment requestTimer timeout
-        else
-            return timeout
+forwardSyncI api0 name responseInfo response timeout priority transId pid = do
     let forwardTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "forward_sync")
             , Erlang.OtpErlangString name
             , Erlang.OtpErlangBinary responseInfo
             , Erlang.OtpErlangBinary response
-            , Erlang.OtpErlangInteger timeoutNew
+            , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangInteger priority
             , Erlang.OtpErlangBinary transId
             , Erlang.OtpErlangPid pid]
@@ -441,6 +457,7 @@ forwardSyncI api0@Instance.T{
             send api0 forwardBinary
             return $ Right api0
 
+-- | forwards a synchronous service request to a different service name
 forwardSync :: Typeable s => Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO ()
@@ -453,6 +470,7 @@ forwardSync api0 name responseInfo response timeout priority transId pid = do
         Right api1 ->
             Exception.throwIO $ ForwardSync api1
 
+-- | provides a response to a service request
 return_ :: Typeable s =>
     Instance.T s -> Instance.RequestType -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
@@ -463,22 +481,14 @@ return_ api0 Instance.SYNC = returnSync api0
 returnAsyncI :: Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-returnAsyncI api0@Instance.T{
-      Instance.requestTimeoutAdjustment = requestTimeoutAdjustment
-    , Instance.requestTimer = requestTimer
-    , Instance.requestTimeout = requestTimeout}
-    name pattern responseInfo response timeout transId pid = do
-    timeoutNew <- if requestTimeoutAdjustment && timeout == requestTimeout then
-            timeoutAdjustment requestTimer timeout
-        else
-            return timeout
+returnAsyncI api0 name pattern responseInfo response timeout transId pid = do
     let returnTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "return_async")
             , Erlang.OtpErlangString name
             , Erlang.OtpErlangString pattern
             , Erlang.OtpErlangBinary responseInfo
             , Erlang.OtpErlangBinary response
-            , Erlang.OtpErlangInteger timeoutNew
+            , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangBinary transId
             , Erlang.OtpErlangPid pid]
     case Erlang.termToBinary returnTerms (-1) of
@@ -488,6 +498,7 @@ returnAsyncI api0@Instance.T{
             send api0 returnBinary
             return $ Right api0
 
+-- | provides a response to an asynchronous service request
 returnAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO ()
@@ -503,22 +514,14 @@ returnAsync api0 name pattern responseInfo response timeout transId pid = do
 returnSyncI :: Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-returnSyncI api0@Instance.T{
-      Instance.requestTimeoutAdjustment = requestTimeoutAdjustment
-    , Instance.requestTimer = requestTimer
-    , Instance.requestTimeout = requestTimeout}
-    name pattern responseInfo response timeout transId pid = do
-    timeoutNew <- if requestTimeoutAdjustment && timeout == requestTimeout then
-            timeoutAdjustment requestTimer timeout
-        else
-            return timeout
+returnSyncI api0 name pattern responseInfo response timeout transId pid = do
     let returnTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "return_sync")
             , Erlang.OtpErlangString name
             , Erlang.OtpErlangString pattern
             , Erlang.OtpErlangBinary responseInfo
             , Erlang.OtpErlangBinary response
-            , Erlang.OtpErlangInteger timeoutNew
+            , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangBinary transId
             , Erlang.OtpErlangPid pid]
     case Erlang.termToBinary returnTerms (-1) of
@@ -528,6 +531,7 @@ returnSyncI api0@Instance.T{
             send api0 returnBinary
             return $ Right api0
 
+-- | provides a response to a synchronous service request
 returnSync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO ()
@@ -540,10 +544,13 @@ returnSync api0 name pattern responseInfo response timeout transId pid = do
         Right api1 ->
             Exception.throwIO $ ReturnSync api1
 
+-- | blocks to receive an asynchronous service request response
 recvAsync :: Typeable s => Instance.T s ->
     Maybe Int -> Maybe ByteString -> Maybe Bool ->
     IO (Result (ByteString, ByteString, ByteString, Instance.T s))
-recvAsync api0@Instance.T{Instance.timeoutSync = timeoutSync'}
+recvAsync api0@Instance.T{
+      Instance.terminateException = terminateException
+    , Instance.timeoutSync = timeoutSync'}
     timeoutOpt transIdOpt consumeOpt =
     let timeout = fromMaybe timeoutSync' timeoutOpt
         transId = fromMaybe transIdNull transIdOpt
@@ -562,45 +569,63 @@ recvAsync api0@Instance.T{Instance.timeoutSync = timeoutSync'}
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{
                       Instance.responseInfo = responseInfo
                     , Instance.response = response
                     , Instance.transId = transId'}) ->
                     return $ Right (responseInfo, response, transId', api1)
 
+-- | returns the 0-based index of this process in the service instance
 processIndex :: Instance.T s -> Int
 processIndex Instance.T{Instance.processIndex = processIndex'} =
     processIndex'
 
+-- | returns the current process count based on the service configuration
 processCount :: Instance.T s -> Int
 processCount Instance.T{Instance.processCount = processCount'} =
     processCount'
 
+-- | returns the count_process_dynamic maximum count
+-- based on the service configuration
 processCountMax :: Instance.T s -> Int
 processCountMax Instance.T{Instance.processCountMax = processCountMax'} =
     processCountMax'
 
+-- | returns the count_process_dynamic minimum count
+-- based on the service configuration
 processCountMin :: Instance.T s -> Int
 processCountMin Instance.T{Instance.processCountMin = processCountMin'} =
     processCountMin'
 
+-- | returns the service name pattern prefix from the service configuration
 prefix :: Instance.T s -> ByteString
 prefix Instance.T{Instance.prefix = prefix'} =
     prefix'
 
+-- | returns the service initialization timeout
+-- from the service configuration
 timeoutInitialize :: Instance.T s -> Int
 timeoutInitialize Instance.T{Instance.timeoutInitialize = timeoutInitialize'} =
     timeoutInitialize'
 
+-- | returns the default asynchronous service request send timeout
+-- from the service configuration
 timeoutAsync :: Instance.T s -> Int
 timeoutAsync Instance.T{Instance.timeoutAsync = timeoutAsync'} =
     timeoutAsync'
 
+-- | returns the default synchronous service request send timeout
+-- from the service configuration
 timeoutSync :: Instance.T s -> Int
 timeoutSync Instance.T{Instance.timeoutSync = timeoutSync'} =
     timeoutSync'
 
+-- | returns the service termination timeout
+-- based on the service configuration
 timeoutTerminate :: Instance.T s -> Int
 timeoutTerminate Instance.T{Instance.timeoutTerminate = timeoutTerminate'} =
     timeoutTerminate'
@@ -616,17 +641,9 @@ callback :: Typeable s => Instance.T s ->
      Int, Int, ByteString, Source) -> IO (Result (Instance.T s))
 callback api0@Instance.T{
       Instance.state = state
-    , Instance.callbacks = callbacks
-    , Instance.requestTimeoutAdjustment = requestTimeoutAdjustment}
+    , Instance.callbacks = callbacks}
     (requestType, name, pattern, requestInfo, request,
      timeout, priority, transId, pid) = do
-    api1 <- if requestTimeoutAdjustment then do
-            requestTimer <- POSIX.getPOSIXTime
-            return api0{
-                  Instance.requestTimer = requestTimer
-                , Instance.requestTimeout = timeout}
-        else
-            return api0
     let (callbackF, callbacksNew) = case Map.lookup pattern callbacks of
             Nothing ->
                 (nullResponse, callbacks)
@@ -636,25 +653,27 @@ callback api0@Instance.T{
                         (Sequence.drop 0 functionQueue) f
                 in
                 (f, Map.insert pattern functionQueueNew callbacks)
-        api2 = api1{Instance.callbacks = callbacksNew}
+        api1 = api0{Instance.callbacks = callbacksNew}
         empty = ByteString.empty
     callbackResultValue <- Exception.try $ case requestType of
         Instance.ASYNC -> do
             callbackResultAsyncValue <- Exception.try $
                 callbackF requestType name pattern
                 requestInfo request timeout priority transId pid
-                state api2
+                state api1
             case callbackResultAsyncValue of
-                Left (ReturnSync api3) -> do
+                Left (Terminate api2) ->
+                    return $ ReturnI (empty, empty, state, setTerminate api2)
+                Left (ReturnAsync api2) ->
+                    return $ Finished api2
+                Left (ReturnSync api2) -> do
                     printException "Synchronous Call Return Invalid"
-                    return $ Finished api3
-                Left (ReturnAsync api3) ->
-                    return $ Finished api3
-                Left (ForwardSync api3) -> do
+                    return $ Finished (setTerminate api2)
+                Left (ForwardAsync api2) ->
+                    return $ Finished api2
+                Left (ForwardSync api2) -> do
                     printException "Synchronous Call Forward Invalid"
-                    return $ Finished api3
-                Left (ForwardAsync api3) ->
-                    return $ Finished api3
+                    return $ Finished (setTerminate api2)
                 Right (Instance.ResponseInfo (v0, v1, v2, v3)) ->
                     return $ ReturnI (v0, v1, v2, v3)
                 Right (Instance.Response (v0, v1, v2)) ->
@@ -672,18 +691,20 @@ callback api0@Instance.T{
             callbackResultSyncValue <- Exception.try $
                 callbackF requestType name pattern
                 requestInfo request timeout priority transId pid
-                state api2
+                state api1
             case callbackResultSyncValue of
-                Left (ReturnSync api3) ->
-                    return $ Finished api3
-                Left (ReturnAsync api3) -> do
+                Left (Terminate api2) ->
+                    return $ ReturnI (empty, empty, state, setTerminate api2)
+                Left (ReturnSync api2) ->
+                    return $ Finished api2
+                Left (ReturnAsync api2) -> do
                     printException "Asynchronous Call Return Invalid"
-                    return $ Finished api3
-                Left (ForwardSync api3) ->
-                    return $ Finished api3
-                Left (ForwardAsync api3) -> do
+                    return $ Finished (setTerminate api2)
+                Left (ForwardSync api2) ->
+                    return $ Finished api2
+                Left (ForwardAsync api2) -> do
                     printException "Asynchronous Call Forward Invalid"
-                    return $ Finished api3
+                    return $ Finished (setTerminate api2)
                 Right (Instance.ResponseInfo (v0, v1, v2, v3)) ->
                     return $ ReturnI (v0, v1, v2, v3)
                 Right (Instance.Response (v0, v1, v2)) ->
@@ -700,32 +721,32 @@ callback api0@Instance.T{
     callbackResultType <- case callbackResultValue of
         Left exception -> do
             printException $ show (exception :: Exception.SomeException)
-            return $ Finished api2
+            return $ ReturnI (empty, empty, state, api1)
         Right callbackResult ->
             return $ callbackResult
     case requestType of
         Instance.ASYNC ->
             case callbackResultType of
-                Finished api4 ->
-                    return $ Right api4
-                ReturnI (responseInfo, response, state', api4) ->
-                    returnAsyncI api4{Instance.state = state'}
+                Finished api3 ->
+                    return $ Right api3
+                ReturnI (responseInfo, response, state', api3) ->
+                    returnAsyncI api3{Instance.state = state'}
                         name pattern responseInfo response timeout transId pid
                 ForwardI (name', requestInfo', request', timeout', priority',
-                          state', api4) ->
-                    forwardAsyncI api4{Instance.state = state'}
+                          state', api3) ->
+                    forwardAsyncI api3{Instance.state = state'}
                         name' requestInfo' request'
                         timeout' priority' transId pid
         Instance.SYNC ->
             case callbackResultType of
-                Finished api4 ->
-                    return $ Right api4
-                ReturnI (responseInfo, response, state', api4) ->
-                    returnSyncI api4{Instance.state = state'}
+                Finished api3 ->
+                    return $ Right api3
+                ReturnI (responseInfo, response, state', api3) ->
+                    returnSyncI api3{Instance.state = state'}
                         name pattern responseInfo response timeout transId pid
                 ForwardI (name', requestInfo', request', timeout', priority',
-                          state', api4) ->
-                    forwardSyncI api4{Instance.state = state'}
+                          state', api3) ->
+                    forwardSyncI api3{Instance.state = state'}
                         name' requestInfo' request'
                         timeout' priority' transId pid
 
@@ -735,11 +756,8 @@ handleEvents messages api0 external cmd0 = do
     cmd <- if cmd0 == 0 then Get.getWord32host else return cmd0
     case () of
       _ | cmd == messageTerm ->
-            let api1 = api0{
-                      Instance.terminate = True
-                    , Instance.timeout = Just False} in
             if external then
-                return ([], api1)
+                return ([], setTerminate api0)
             else
                 fail terminateError
         | cmd == messageReinit -> do
@@ -747,13 +765,11 @@ handleEvents messages api0 external cmd0 = do
             timeoutAsync' <- Get.getWord32host
             timeoutSync' <- Get.getWord32host
             priorityDefault <- Get.getInt8
-            requestTimeoutAdjustment <- Get.getWord8
             let api1 = Instance.reinit api0
                     processCount'
                     timeoutAsync'
                     timeoutSync'
                     priorityDefault
-                    requestTimeoutAdjustment
             empty <- Get.isEmpty
             if not empty then
                 handleEvents messages api1 external 0
@@ -787,7 +803,6 @@ pollRequestDataGet messages api0 external = do
             timeoutSync' <- Get.getWord32host
             timeoutTerminate' <- Get.getWord32host
             priorityDefault <- Get.getInt8
-            requestTimeoutAdjustment <- Get.getWord8
             let api1 = Instance.init api0
                     processIndex'
                     processCount'
@@ -799,7 +814,6 @@ pollRequestDataGet messages api0 external = do
                     timeoutSync'
                     timeoutTerminate'
                     priorityDefault
-                    requestTimeoutAdjustment
             empty <- Get.isEmpty
             if not empty then
                 handleEvents messages api1 external 0
@@ -893,13 +907,11 @@ pollRequestDataGet messages api0 external = do
             timeoutAsync' <- Get.getWord32host
             timeoutSync' <- Get.getWord32host
             priorityDefault <- Get.getInt8
-            requestTimeoutAdjustment <- Get.getWord8
             let api1 = Instance.reinit api0
                     processCount'
                     timeoutAsync'
                     timeoutSync'
                     priorityDefault
-                    requestTimeoutAdjustment
             empty <- Get.isEmpty
             if not empty then
                 pollRequestDataGet messages api1 external
@@ -949,16 +961,15 @@ pollRequestData api0 external dataIn =
 pollRequestLoop :: Typeable s =>
     Instance.T s -> Int -> Bool -> Clock.NominalDiffTime ->
     IO (Result (Bool, Instance.T s))
-pollRequestLoop api0@Instance.T{
-      Instance.socketHandle = socketHandle
-    , Instance.bufferRecvSize = bufferRecvSize}
-    timeout external pollTimer = do
-    inputAvailable <- if bufferRecvSize > 0 then
-            return True
-        else
-            SysIO.hWaitForInput socketHandle timeout
-    if not inputAvailable then
+pollRequestLoop api0 timeout external pollTimer = do
+    inputAvailable <- pollWait api0 timeout
+    if inputAvailable == Just False then
         return $ Right (True, api0{Instance.timeout = Nothing})
+    else if inputAvailable == Nothing then
+        let api1 = api0{
+              Instance.terminate = True
+            , Instance.timeout = Nothing} in
+        return $ Right (False, api1)
     else do
         (dataIn, _, api1) <- recv api0
         dataResult <- pollRequestData api1 external dataIn
@@ -994,7 +1005,10 @@ pollRequest api0@Instance.T{
       Instance.initializationComplete = initializationComplete
     , Instance.terminate = terminate} timeout external =
     if terminate then
-        return $ Right (False, api0)
+        if external then
+            return $ Right (False, api0)
+        else
+            return $ Left terminateError
     else do
         pollTimer <- if timeout <= 0 then
                 return 0
@@ -1013,10 +1027,44 @@ pollRequest api0@Instance.T{
         else
             pollRequestLoopBegin api0 timeout external pollTimer
 
+pollWait :: Typeable s => Instance.T s -> Int -> IO (Maybe Bool)
+pollWait Instance.T{
+      Instance.socketHandle = socketHandle
+    , Instance.bufferRecvSize = bufferRecvSize}
+    timeout = do
+    if bufferRecvSize > 0 then
+        return $ Just True
+    else
+        SysIOErr.catchIOError
+            (do
+             inputAvailable <- SysIO.hWaitForInput socketHandle timeout
+             return $ Just inputAvailable)
+            (\e -> if SysIOErr.isEOFError e then
+                 return Nothing
+             else
+                 SysIOErr.ioError e)
+
+-- | blocks to process incoming CloudI service requests
 poll :: Typeable s => Instance.T s -> Int -> IO (Result (Bool, Instance.T s))
 poll api0 timeout =
     pollRequest api0 timeout True
 
+-- | shutdown the service successfully
+shutdown :: Instance.T s -> Maybe ByteString ->
+    IO (Result (Instance.T s))
+shutdown api0 reasonOpt =
+    let reason = fromMaybe Char8.empty reasonOpt
+        shutdownTerms = Erlang.OtpErlangTuple
+            [ Erlang.OtpErlangAtom (Char8.pack "shutdown")
+            , Erlang.OtpErlangString reason]
+    in
+    case Erlang.termToBinary shutdownTerms (-1) of
+        Left err ->
+            return $ Left $ show err
+        Right shutdownBinary -> do
+            send api0 shutdownBinary
+            return $ Right $ api0
+
 send :: Instance.T s -> LazyByteString -> IO ()
 send Instance.T{
       Instance.useHeader = True
@@ -1101,20 +1149,6 @@ recv api0@Instance.T{
               Instance.bufferRecv = Monoid.mempty
             , Instance.bufferRecvSize = 0})
 
-timeoutAdjustment :: Clock.NominalDiffTime -> Int ->
-    IO (Int)
-timeoutAdjustment t0 timeout = do
-    t1 <- POSIX.getPOSIXTime
-    if t1 <= t0 then
-        return timeout
-    else
-        let elapsed = floor ((t1 - t0) * 1000) :: Integer
-            timeoutValue = fromIntegral timeout :: Integer in
-        if elapsed >= timeoutValue then
-            return 0
-        else
-            return $ fromIntegral $ timeoutValue - elapsed
-
 timeoutAdjustmentPoll :: Clock.NominalDiffTime -> Int ->
     IO (Clock.NominalDiffTime, Int)
 timeoutAdjustmentPoll t0 timeout = do
@@ -1129,9 +1163,19 @@ timeoutAdjustmentPoll t0 timeout = do
         else
             return (t1, fromIntegral $ timeoutValue - elapsed)
 
+setTerminate :: Instance.T s -> Instance.T s
+setTerminate api0 =
+    api0{ Instance.terminate = True
+        , Instance.timeout = Just False}
+
 threadList :: Concurrent.MVar [Concurrent.MVar ()]
 threadList = Unsafe.unsafePerformIO (Concurrent.newMVar [])
 
+-- | simplifies thread creation and join
+--
+-- > Concurrent.setNumCapabilities threadCount
+-- > mapM_ (CloudI.threadCreate task) [0..threadCount - 1]
+-- > CloudI.threadsWait
 threadCreate :: (Int -> IO ()) -> Int -> IO ThreadId
 threadCreate f threadIndex = do
     thread <- Concurrent.newEmptyMVar
@@ -1148,6 +1192,7 @@ threadCreateFork threadIndex action afterF =
         Concurrent.forkOn threadIndex
             (Exception.try (restore action) >>= afterF)
 
+-- | wait for threads to join after being created by 'threadCreate'
 threadsWait :: IO ()
 threadsWait = do
     threads <- Concurrent.takeMVar threadList
@@ -1159,8 +1204,8 @@ threadsWait = do
             Concurrent.takeMVar done
             threadsWait
 
-textKeyValueParse :: ByteString -> Map ByteString [ByteString]
-textKeyValueParse text =
+textPairsParse :: ByteString -> Map ByteString [ByteString]
+textPairsParse text =
     let loop m [] = m
         loop m [v] =
             if v == ByteString.empty then
@@ -1172,10 +1217,31 @@ textKeyValueParse text =
                 Nothing ->
                     loop (Map.insert k [v] m) l'
                 Just v' ->
-                    loop (Map.insert k (v:v') m) l'
+                    loop (Map.insert k (v' ++ [v]) m) l'
     in
     loop Map.empty (Char8.split '\0' text)
 
+textPairsNew :: Map ByteString [ByteString] -> Maybe Bool -> ByteString
+textPairsNew pairs responseOpt =
+    let response = fromMaybe True responseOpt in
+    if response && Map.size pairs == 0 then
+        Char8.pack "\0"
+    else
+        let pair builder _ [] =
+                builder
+            pair builder k (v:l') =
+                pair (builder <>
+                    Builder.byteString k <> Builder.char8 '\0' <>
+                    Builder.byteString v <> Builder.char8 '\0') k l'
+        in
+        LazyByteString.toStrict $ Builder.toLazyByteString $
+            Map.foldlWithKey pair Monoid.mempty pairs
+
+-- | decode service request info key/value data
 infoKeyValueParse :: ByteString -> Map ByteString [ByteString]
-infoKeyValueParse = textKeyValueParse
+infoKeyValueParse = textPairsParse
+
+-- | encode service response info key/value data
+infoKeyValueNew :: Map ByteString [ByteString] -> Maybe Bool -> ByteString
+infoKeyValueNew = textPairsNew
 
