@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -33,12 +33,14 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.PrintStream;
 import java.io.UnsupportedEncodingException;
-import java.util.HashMap;
-import java.util.LinkedList;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.UUID;
 import java.util.Date;
+import java.util.HashMap;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.UUID;
 import java.text.SimpleDateFormat;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ExecutorService;
@@ -179,7 +181,7 @@ public class API
         this.timeout_initialize = 5000;
         this.timeout_async = 5000;
         this.timeout_sync = 5000;
-        this.timeout_terminate = 1000; // TIMEOUT_TERMINATE_MIN
+        this.timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
         this.priority_default = 0;
 
         // send the initialization message to the managing Erlang process
@@ -1067,11 +1069,17 @@ public class API
                 }
                 else if (response.getClass() == byte[][].class)
                 {
-                    byte [][] response_array = (byte[][]) response;
+                    final byte [][] response_array = (byte[][]) response;
                     assert response_array.length == 2 : "invalid response";
+                    byte [] response_info_value = response_array[0];
+                    byte [] response_value = response_array[1];
+                    if (response_info_value == null)
+                        response_info_value = ("").getBytes();
+                    if (response_value == null)
+                        response_value = ("").getBytes();
                     return_async(name, pattern,
-                                 response_array[0],
-                                 response_array[1],
+                                 response_info_value,
+                                 response_value,
                                  timeout, trans_id, pid);
 
                 }
@@ -1091,41 +1099,48 @@ public class API
                 }
                 return;
             }
-            catch (InvalidInputException e)
-            {
-                throw e;
-            }
             catch (MessageDecodingException e)
             {
-                throw e;
+                this.terminate = true;
             }
             catch (TerminateException e)
             {
-                throw e;
             }
             catch (ReturnAsyncException e_return)
             {
                 return;
             }
+            catch (ReturnSyncException e_return)
+            {
+                this.terminate = true;
+                e_return.printStackTrace(API.err);
+                return;
+            }
             catch (ForwardAsyncException e_forward)
             {
                 return;
             }
+            catch (ForwardSyncException e_forward)
+            {
+                this.terminate = true;
+                e_forward.printStackTrace(API.err);
+                return;
+            }
             catch (Throwable e)
             {
                 e.printStackTrace(API.err);
-                try
-                {
-                    return_async(name, pattern,
-                                 ("").getBytes(),
-                                 ("").getBytes(),
-                                 timeout, trans_id, pid);
-                }
-                catch (ReturnAsyncException e_return)
-                {
-                }
-                return;
             }
+            try
+            {
+                return_async(name, pattern,
+                             ("").getBytes(),
+                             ("").getBytes(),
+                             timeout, trans_id, pid);
+            }
+            catch (ReturnAsyncException e_return)
+            {
+            }
+            return;
         }
         else if (command == MESSAGE_SEND_SYNC)
         {
@@ -1144,11 +1159,17 @@ public class API
                 }
                 else if (response.getClass() == byte[][].class)
                 {
-                    byte [][] response_array = (byte[][]) response;
+                    final byte [][] response_array = (byte[][]) response;
                     assert response_array.length == 2 : "invalid response";
+                    byte [] response_info_value = response_array[0];
+                    byte [] response_value = response_array[1];
+                    if (response_info_value == null)
+                        response_info_value = ("").getBytes();
+                    if (response_value == null)
+                        response_value = ("").getBytes();
                     return_sync(name, pattern,
-                                response_array[0],
-                                response_array[1],
+                                response_info_value,
+                                response_value,
                                 timeout, trans_id, pid);
 
                 }
@@ -1168,41 +1189,48 @@ public class API
                 }
                 return;
             }
-            catch (InvalidInputException e)
-            {
-                throw e;
-            }
             catch (MessageDecodingException e)
             {
-                throw e;
+                this.terminate = true;
             }
             catch (TerminateException e)
             {
-                throw e;
             }
             catch (ReturnSyncException e_return)
             {
                 return;
             }
+            catch (ReturnAsyncException e_return)
+            {
+                this.terminate = true;
+                e_return.printStackTrace(API.err);
+                return;
+            }
             catch (ForwardSyncException e_forward)
             {
                 return;
             }
+            catch (ForwardAsyncException e_forward)
+            {
+                this.terminate = true;
+                e_forward.printStackTrace(API.err);
+                return;
+            }
             catch (Throwable e)
             {
                 e.printStackTrace(API.err);
-                try
-                {
-                    return_sync(name, pattern,
-                                ("").getBytes(),
-                                ("").getBytes(),
-                                timeout, trans_id, pid);
-                }
-                catch (ReturnSyncException e_return)
-                {
-                }
-                return;
             }
+            try
+            {
+                return_sync(name, pattern,
+                            ("").getBytes(),
+                            ("").getBytes(),
+                            timeout, trans_id, pid);
+            }
+            catch (ReturnSyncException e_return)
+            {
+            }
+            return;
         }
         else
         {
@@ -1274,7 +1302,10 @@ public class API
     {
         if (this.terminate)
         {
-            return Boolean.FALSE;
+            if (external)
+                return Boolean.FALSE;
+            else
+                throw new TerminateException(this.timeout_terminate);
         }
         else if (external && ! this.initialization_complete)
         {
@@ -1355,6 +1386,8 @@ public class API
                         }
                         callback(command, name, pattern, request_info, request,
                                  request_timeout, priority, trans_id, pid);
+                        if (this.terminate)
+                            return Boolean.FALSE;
                         break;
                     }
                     case MESSAGE_RECV_ASYNC:
@@ -1521,7 +1554,7 @@ public class API
     /**
      * Shutdown the service successfully
      *
-     * @param  reason      the shutdown reason
+     * @param  reason  the shutdown reason
      */
     public void shutdown(final String reason)
     {
@@ -1533,31 +1566,31 @@ public class API
         send(shutdown);
     }
 
-    private HashMap<String,
-                    ArrayList<String>> binary_key_value_parse(final byte[] b)
+    private static HashMap<String,
+                           ArrayList<String>> text_pairs_parse(final byte[] t)
     {
-        HashMap<String, ArrayList<String> > result =
-            new HashMap<String, ArrayList<String> >();
+        final HashMap<String, ArrayList<String>> pairs =
+            new HashMap<String, ArrayList<String>>();
         String key = null;
-        int binary_i = 0;
-        for (int binary_j = 0; binary_j < b.length; ++binary_j)
+        int text_i = 0;
+        for (int text_j = 0; text_j < t.length; ++text_j)
         {
-            if (b[binary_j] == 0)
+            if (t[text_j] == 0)
             {
                 if (key == null)
                 {
-                    key = new String(b, binary_i, binary_j - binary_i);
+                    key = new String(t, text_i, text_j - text_i);
                 }
                 else
                 {
-                    ArrayList<String> value = result.get(key);
-                    final String element = new String(b, binary_i,
-                                                      binary_j - binary_i);
+                    ArrayList<String> value = pairs.get(key);
+                    final String element = new String(t, text_i,
+                                                      text_j - text_i);
                     if (value == null)
                     {
                         value = new ArrayList<String>();
                         value.add(element);
-                        result.put(key, value);
+                        pairs.put(key, value);
                     }
                     else
                     {
@@ -1565,16 +1598,67 @@ public class API
                     }
                     key = null;
                 }
-                binary_i = binary_j + 1;
+                text_i = text_j + 1;
             }
         }
-        return result;
+        return pairs;
     }
 
-    public HashMap<String,
-                   ArrayList<String>> info_key_value_parse(final byte[] info)
+    private static byte[] text_pairs_new(final Map<String, List<String>> pairs,
+                                         final boolean response)
     {
-        return binary_key_value_parse(info);
+        final ByteArrayOutputStream text = new ByteArrayOutputStream(1024);
+        for (Map.Entry<String, List<String>> pair : pairs.entrySet())
+        {
+            final byte[] key_bytes = pair.getKey().getBytes();
+            for (String value : pair.getValue())
+            {
+                final byte[] value_bytes = value.getBytes();
+                text.write(key_bytes, 0, key_bytes.length);
+                text.write(0);
+                text.write(value_bytes, 0, value_bytes.length);
+                text.write(0);
+            }
+        }
+        if (response && text.size() == 0)
+            text.write(0);
+        return text.toByteArray();
+    }
+
+    /**
+     * Decode service request info key/value data
+     *
+     * @param  info encoded binary
+     * @return info key/value map
+     */
+    public static HashMap<String,
+                          ArrayList<String>> info_key_value_parse(byte[] info)
+    {
+        return API.text_pairs_parse(info);
+    }
+
+    /**
+     * Encode service response info key/value data
+     *
+     * @param  pairs info key/value map
+     * @return encoded binary
+     */
+    public static byte[] info_key_value_new(Map<String, List<String>> pairs)
+    {
+        return API.info_key_value_new(pairs, true);
+    }
+
+    /**
+     * Encode service response info key/value data
+     *
+     * @param  pairs info key/value map
+     * @param  response if encoding response data
+     * @return encoded binary
+     */
+    public static byte[] info_key_value_new(Map<String, List<String>> pairs,
+                                            boolean response)
+    {
+        return API.text_pairs_new(pairs, response);
     }
 
     private void send(final OtpOutputStream command)
