@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '1.80';
+$CloudI::API::VERSION = '2.00';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -71,7 +71,7 @@ sub new
 {
     my $class = shift;
     my ($thread_index) = @_;
-    if (! defined($thread_index) || $thread_index !~ /^\d+$/)
+    if (! defined($thread_index) or $thread_index !~ /^\d+$/)
     {
         die CloudI::InvalidInputException->new();
     }
@@ -81,7 +81,7 @@ sub new
         die CloudI::InvalidInputException->new();
     }
     my $buffer_size = getenv('CLOUDI_API_INIT_BUFFER_SIZE');
-    if (! defined($buffer_size) || $buffer_size !~ /^\d+$/)
+    if (! defined($buffer_size) or $buffer_size !~ /^\d+$/)
     {
         die CloudI::InvalidInputException->new();
     }
@@ -120,7 +120,7 @@ sub new
         _timeout_initialize => undef,
         _timeout_async => undef,
         _timeout_sync => undef,
-        _timeout_terminate => 1000, # TIMEOUT_TERMINATE_MIN
+        _timeout_terminate => 10, # TIMEOUT_TERMINATE_MIN
         _priority_default => undef,
     }, $class;
     $self->_send(Erlang::term_to_binary(Erlang::OtpErlangAtom->new('init')));
@@ -150,7 +150,7 @@ sub DESTROY
 sub thread_count
 {
     my $count = getenv('CLOUDI_API_INIT_THREAD_COUNT');
-    if (! defined($count) || $count !~ /^\d+$/)
+    if (! defined($count) or $count !~ /^\d+$/)
     {
         die CloudI::InvalidInputException->new();
     }
@@ -163,12 +163,12 @@ sub subscribe
     # arity 2 or 3 (so using an object is not required)
     my ($pattern, $object, $method) = @_;
     my $function;
-    if (! defined($method) && ref($object) eq 'CODE')
+    if (! defined($method) and ref($object) eq 'CODE')
     {
         $function = $object;
     }
-    elsif (defined(blessed($object)) &&
-           defined($method) && ref($method) eq '' &&
+    elsif (defined(blessed($object)) and
+           defined($method) and ref($method) eq '' and
            defined($object->can($method)))
     {
         # object method
@@ -486,10 +486,10 @@ sub _callback
         $function = shift(@function_queue);
         push(@function_queue, $function);
     }
+    my $response_info;
+    my $response;
     if ($command == MESSAGE_SEND_ASYNC)
     {
-        my $response_info;
-        my $response;
         eval
         {
             ($response_info,
@@ -505,7 +505,7 @@ sub _callback
                     $response = '';
                 }
             }
-            elsif (ref($response_info) ne '' ||
+            elsif (ref($response_info) ne '' or
                    ! defined($response_info))
             {
                 $response_info = '';
@@ -518,30 +518,31 @@ sub _callback
         my $e = $@;
         if ($e)
         {
-            if ($e->isa('CloudI::InvalidInputException') ||
-                $e->isa('CloudI::MessageDecodingException') ||
-                $e->isa('CloudI::TerminateException'))
+            if ($e->isa('CloudI::MessageDecodingException'))
             {
-                die $e;
+                $self->{_terminate} = 1;
             }
-            elsif ($e->isa('CloudI::ReturnAsyncException') ||
+            elsif ($e->isa('CloudI::TerminateException'))
+            {
+            }
+            elsif ($e->isa('CloudI::ReturnAsyncException') or
                    $e->isa('CloudI::ForwardAsyncException'))
             {
                 return;
             }
-            elsif ($e->isa('CloudI::ReturnSyncException') ||
+            elsif ($e->isa('CloudI::ReturnSyncException') or
                    $e->isa('CloudI::ForwardSyncException'))
             {
+                $self->{_terminate} = 1;
                 print "$e";
-                assert(0);
                 return;
             }
             else
             {
                 print "$e";
-                $response_info = '';
-                $response = '';
             }
+            $response_info = '';
+            $response = '';
         }
         eval
         {
@@ -560,8 +561,6 @@ sub _callback
     }
     elsif ($command == MESSAGE_SEND_SYNC)
     {
-        my $response_info;
-        my $response;
         eval
         {
             ($response_info,
@@ -577,7 +576,7 @@ sub _callback
                     $response = '';
                 }
             }
-            elsif (ref($response_info) ne '' ||
+            elsif (ref($response_info) ne '' or
                    ! defined($response_info))
             {
                 $response_info = '';
@@ -590,35 +589,36 @@ sub _callback
         my $e = $@;
         if ($e)
         {
-            if ($e->isa('CloudI::InvalidInputException') ||
-                $e->isa('CloudI::MessageDecodingException') ||
-                $e->isa('CloudI::TerminateException'))
+            if ($e->isa('CloudI::MessageDecodingException'))
             {
-                die $e;
+                $self->{_terminate} = 1;
             }
-            elsif ($e->isa('CloudI::ReturnSyncException') ||
+            elsif ($e->isa('CloudI::TerminateException'))
+            {
+            }
+            elsif ($e->isa('CloudI::ReturnSyncException') or
                    $e->isa('CloudI::ForwardSyncException'))
             {
                 return;
             }
-            elsif ($e->isa('CloudI::ReturnAsyncException') ||
+            elsif ($e->isa('CloudI::ReturnAsyncException') or
                    $e->isa('CloudI::ForwardAsyncException'))
             {
+                $self->{_terminate} = 1;
                 print "$e";
-                assert(0);
                 return;
             }
             else
             {
                 print "$e";
-                $response_info = '';
-                $response = '';
             }
+            $response_info = '';
+            $response = '';
         }
         eval
         {
             $self->return_sync($name, $pattern, $response_info, $response,
-                                $timeout, $trans_id, $pid);
+                               $timeout, $trans_id, $pid);
         };
         $e = $@;
         if ($e)
@@ -634,7 +634,6 @@ sub _callback
     {
         die CloudI::MessageDecodingException->new();
     }
-
 }
 
 sub _handle_events
@@ -709,9 +708,16 @@ sub _poll_request
     my ($timeout, $external) = @_;
     if ($self->{_terminate})
     {
-        return 0;
+        if ($external)
+        {
+            return 0;
+        }
+        else
+        {
+            die CloudI::TerminateException->new($self->{_timeout_terminate});
+        }
     }
-    elsif ($external && ! $self->{_initialization_complete})
+    elsif ($external and ! $self->{_initialization_complete})
     {
         $self->_send(Erlang::term_to_binary(
             Erlang::OtpErlangAtom->new('polling')));
@@ -720,7 +726,7 @@ sub _poll_request
 
     my $poll_timer;
     my $timeout_value_secs;
-    if (! defined($timeout) || $timeout < 0)
+    if (! defined($timeout) or $timeout < 0)
     {
         $timeout_value_secs = undef;
     }
@@ -739,7 +745,7 @@ sub _poll_request
     vec($result_except, fileno($self->{_s}), 1) = 1;
     my $result = select($result_read, undef, $result_except,
                         $timeout_value_secs);
-    if ($result == -1 || vec($result_except, fileno($self->{_s}), 1) == 1)
+    if ($result == -1 or vec($result_except, fileno($self->{_s}), 1) == 1)
     {
         return 0;
     }
@@ -787,7 +793,7 @@ sub _poll_request
                     $timeout_sync, $timeout_async,
                     $timeout_terminate, $priority_default);
         }
-        elsif ($command == MESSAGE_SEND_ASYNC ||
+        elsif ($command == MESSAGE_SEND_ASYNC or
                $command == MESSAGE_SEND_SYNC)
         {
             $i += $j; $j = 4;
@@ -826,8 +832,12 @@ sub _poll_request
                              $request_info, $request,
                              $request_timeout, $priority, $trans_id,
                              Erlang::binary_to_term($pid));
+            if ($self->{_terminate})
+            {
+                return 0;
+            }
         }
-        elsif ($command == MESSAGE_RECV_ASYNC ||
+        elsif ($command == MESSAGE_RECV_ASYNC or
                $command == MESSAGE_RETURN_SYNC)
         {
             $i += $j; $j = 4;
@@ -972,7 +982,7 @@ sub _poll_request
         vec($result_except, fileno($self->{_s}), 1) = 1;
         $result = select($result_read, undef, $result_except,
                          $timeout_value_secs);
-        if ($result == -1 || vec($result_except, fileno($self->{_s}), 1) == 1)
+        if ($result == -1 or vec($result_except, fileno($self->{_s}), 1) == 1)
         {
             return 0;
         }
@@ -1014,46 +1024,84 @@ sub shutdown
         Erlang::OtpErlangAtom->new('shutdown'), $reason]));
 }
 
-sub _text_key_value_parse
+# callable without an object
+sub _text_pairs_parse
 {
-    my $self = shift;
     my ($text) = @_;
-    my %result = ();
-    my @data = split("\0", $text);
-    my $size = scalar(@data);
+    my %pairs = ();
+    my @text_segments = split("\0", $text);
+    my $size = scalar(@text_segments);
     if ($size >= 2)
     {
         use integer;
         for my $i_step (0 .. (($size / 2) - 1))
         {
             my $i = $i_step * 2;
-            my $key = $data[$i];
-            my $value = $result{$key};
+            my $key = $text_segments[$i];
+            my $value = $pairs{$key};
             if (defined($value))
             {
                 if (ref($value) eq 'ARRAY')
                 {
-                    push(@$value, $data[$i + 1]);
+                    push(@$value, $text_segments[$i + 1]);
                 }
                 else
                 {
-                    $result{$key} = [$value, $data[$i + 1]];
+                    $pairs{$key} = [$value, $text_segments[$i + 1]];
                 }
             }
             else
             {
-                $result{$key} = $data[$i + 1];
+                $pairs{$key} = $text_segments[$i + 1];
             }
         }
     }
-    return %result;
+    return %pairs;
 }
 
+# callable without an object
+sub _text_pairs_new
+{
+    my ($pairs_ref, $response) = @_;
+    my $text = '';
+    while (my ($key, $values) = each(%$pairs_ref))
+    {
+        if (ref($values) eq '' and scalar($values) eq $values)
+        {
+            # values is a string
+            $text .= $key . "\0" . $values . "\0";
+        }
+        else
+        {
+            for my $value (@$values)
+            {
+                $text .= $key . "\0" . $value . "\0";
+            }
+        }
+    }
+    if ($response and $text eq '')
+    {
+        $text = "\0";
+    }
+    return $text;
+}
+
+# callable without an object
 sub info_key_value_parse
 {
-    my $self = shift;
-    my ($message_info) = @_;
-    return $self->_text_key_value_parse($message_info);
+    my ($info) = @_;
+    return _text_pairs_parse($info);
+}
+
+# callable without an object
+sub info_key_value_new
+{
+    my ($pairs_ref, $response) = @_;
+    if (! defined($response))
+    {
+        $response = 1;
+    }
+    return _text_pairs_new($pairs_ref, $response);
 }
 
 sub _send
@@ -1080,7 +1128,7 @@ sub _recv
         while ($i < 4)
         {
             $read = sysread($self->{_s}, $fragment, 4 - $i);
-            if (! defined($read) || $read == 0)
+            if (! defined($read) or $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
             }
@@ -1094,7 +1142,7 @@ sub _recv
         {
             $read = sysread($self->{_s}, $fragment,
                             _min($total - $i, $self->{_size}));
-            if (! defined($read) || $read == 0)
+            if (! defined($read) or $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
             }
@@ -1110,7 +1158,7 @@ sub _recv
         while ($ready)
         {
             $read = sysread($self->{_s}, $fragment, $self->{_size});
-            if (! defined($read) || $read == 0)
+            if (! defined($read) or $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
             }
