@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -89,7 +89,7 @@ class API
         $this->terminate = false;
         $this->size = intval($buffer_size_str);
         $this->callbacks = array();
-        $this->timeout_terminate = 1000; // TIMEOUT_TERMINATE_MIN
+        $this->timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
         $this->send(\Erlang\term_to_binary(new \Erlang\OtpErlangAtom('init')));
         list($this->process_index,
              $this->process_count,
@@ -351,6 +351,7 @@ class API
             $function = array_shift($function_queue);
             $function_queue[] = $function;
         }
+        $return_null_response = false;
         switch ($command)
         {
             case MESSAGE_SEND_ASYNC:
@@ -376,41 +377,42 @@ class API
                     if (! is_string($response))
                         $response = '';
                 }
-                catch (InvalidInputException $e)
-                {
-                    throw $e;
-                }
                 catch (MessageDecodingException $e)
                 {
-                    throw $e;
+                    $this->terminate = true;
+                    $return_null_response = true;
                 }
                 catch (TerminateException $e)
                 {
-                    throw $e;
+                    $return_null_response = true;
                 }
                 catch (ReturnAsyncException $e)
                 {
                     return;
                 }
+                catch (ReturnSyncException $e)
+                {
+                    $this->terminate = true;
+                    echo "{$e->getMessage()}\n{$e}\n";
+                    return;
+                }
                 catch (ForwardAsyncException $e)
                 {
                     return;
                 }
-                catch (ReturnSyncException $e)
-                {
-                    echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
-                    return;
-                }
                 catch (ForwardSyncException $e)
                 {
+                    $this->terminate = true;
                     echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
                     return;
                 }
                 catch (\Exception $e)
                 {
+                    $return_null_response = true;
                     echo "{$e->getMessage()}\n{$e}\n";
+                }
+                if ($return_null_response)
+                {
                     $response_info = '';
                     $response = '';
                 }
@@ -447,41 +449,42 @@ class API
                     if (! is_string($response))
                         $response = '';
                 }
-                catch (InvalidInputException $e)
-                {
-                    throw $e;
-                }
                 catch (MessageDecodingException $e)
                 {
-                    throw $e;
+                    $this->terminate = true;
+                    $return_null_response = true;
                 }
                 catch (TerminateException $e)
                 {
-                    throw $e;
+                    $return_null_response = true;
                 }
                 catch (ReturnSyncException $e)
                 {
                     return;
                 }
+                catch (ReturnAsyncException $e)
+                {
+                    $this->terminate = true;
+                    echo "{$e->getMessage()}\n{$e}\n";
+                    return;
+                }
                 catch (ForwardSyncException $e)
                 {
                     return;
                 }
-                catch (ReturnAsyncException $e)
-                {
-                    echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
-                    return;
-                }
                 catch (ForwardAsyncException $e)
                 {
+                    $this->terminate = true;
                     echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
                     return;
                 }
                 catch (\Exception $e)
                 {
+                    $return_null_response = true;
                     echo "{$e->getMessage()}\n{$e}\n";
+                }
+                if ($return_null_response)
+                {
                     $response_info = '';
                     $response = '';
                 }
@@ -554,7 +557,10 @@ class API
     {
         if ($this->terminate)
         {
-            return false;
+            if ($external)
+                return false;
+            else
+                throw new TerminateException($this->timeout_terminate);
         }
         elseif ($external && ! $this->initialization_complete)
         {
@@ -676,6 +682,10 @@ class API
                                     $request_info, $request,
                                     $request_timeout, $priority, $trans_id,
                                     \Erlang\binary_to_term($pid));
+                    if ($this->terminate)
+                    {
+                        return false;
+                    }
                     break;
                 case MESSAGE_RECV_ASYNC:
                 case MESSAGE_RETURN_SYNC:
@@ -833,36 +843,63 @@ class API
             array(new \Erlang\OtpErlangAtom('shutdown'), $reason)));
     }
 
-    private function text_key_value_parse($text)
+    private static function text_pairs_parse($text)
     {
-        $result = array();
-        $data = explode("\0", $text);
-        $size = count($data);
+        $pairs = array();
+        $text_segments = explode("\0", $text);
+        $size = count($text_segments);
         if ($size >= 2)
         {
             foreach (range(0, ($size - $size % 2) - 2, 2) as $i)
             {
-                $key = $data[$i];
-                if (isset($result[$key]))
+                $key = $text_segments[$i];
+                if (isset($pairs[$key]))
                 {
-                    $value = $result[$key];
+                    $value = $pairs[$key];
                     if (is_array($value))
-                        $value[] = $data[$i + 1];
+                        $value[] = $text_segments[$i + 1];
                     else
-                        $result[$key] = array($value, $data[$i + 1]);
+                        $pairs[$key] = array($value, $text_segments[$i + 1]);
                 }
                 else
                 {
-                    $result[$key] = $data[$i + 1];
+                    $pairs[$key] = $text_segments[$i + 1];
                 }
             }
         }
-        return $result;
+        return $pairs;
     }
 
-    public function info_key_value_parse($message_info)
+    private static function text_pairs_new($pairs, $response)
     {
-        return $this->text_key_value_parse($message_info);
+        $text = '';
+        foreach ($pairs as $key => $values)
+        {
+            if (is_string($values))
+            {
+                $text .= $key . "\0" . $values . "\0";
+            }
+            else
+            {
+                foreach ($values as $value)
+                {
+                    $text .= $key . "\0" . $value . "\0";
+                }
+            }
+        }
+        if ($response && empty($text))
+            $text = "\0";
+        return $text;
+    }
+
+    public static function info_key_value_parse($info)
+    {
+        return self::text_pairs_parse($info);
+    }
+
+    public static function info_key_value_new($pairs, $response = true)
+    {
+        return self::text_pairs_new($pairs, $response);
     }
 
     private function send($data)
