@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+# Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -31,6 +31,7 @@ $stdout.sync = true
 $stderr.sync = true
 
 require 'erlang'
+require 'stringio'
 
 module CloudI
     class API
@@ -62,7 +63,7 @@ module CloudI
             @terminate = false
             @size = buffer_size_str.to_i
             @callbacks = Hash.new
-            @timeout_terminate = 1000 # TIMEOUT_TERMINATE_MIN
+            @timeout_terminate = 10 # TIMEOUT_TERMINATE_MIN
             send(Erlang.term_to_binary(:init))
             poll_request(nil, false)
         end
@@ -253,6 +254,7 @@ module CloudI
                 function = function_queue.shift
                 function_queue.push(function)
             end
+            return_null_response = false
             case command
             when MESSAGE_SEND_ASYNC
                 begin
@@ -272,29 +274,31 @@ module CloudI
                     if not response.kind_of?(String)
                         response = ''
                     end
-                rescue InvalidInputException => e
-                    raise e
                 rescue MessageDecodingException => e
-                    raise e
+                    @terminate = true
+                    return_null_response = true
                 rescue TerminateException => e
-                    raise e
+                    return_null_response = true
                 rescue ReturnAsyncException
                     return
+                rescue ReturnSyncException => e
+                    @terminate = true
+                    $stderr.puts e.message
+                    $stderr.puts e.backtrace
+                    return
                 rescue ForwardAsyncException
                     return
-                rescue ReturnSyncException => e
-                    $stderr.puts e.message
-                    $stderr.puts e.backtrace
-                    API.assert{false}
-                    return
                 rescue ForwardSyncException => e
+                    @terminate = true
                     $stderr.puts e.message
                     $stderr.puts e.backtrace
-                    API.assert{false}
                     return
                 rescue
+                    return_null_response = true
                     $stderr.puts $!.message
                     $stderr.puts $!.backtrace
+                end
+                if return_null_response
                     response_info = ''
                     response = ''
                 end
@@ -321,29 +325,31 @@ module CloudI
                     if not response.kind_of?(String)
                         response = ''
                     end
-                rescue InvalidInputException => e
-                    raise e
                 rescue MessageDecodingException => e
-                    raise e
+                    @terminate = true
+                    return_null_response = true
                 rescue TerminateException => e
-                    raise e
+                    return_null_response = true
                 rescue ReturnSyncException
                     return
+                rescue ReturnAsyncException => e
+                    @terminate = true
+                    $stderr.puts e.message
+                    $stderr.puts e.backtrace
+                    return
                 rescue ForwardSyncException
                     return
-                rescue ReturnAsyncException => e
-                    $stderr.puts e.message
-                    $stderr.puts e.backtrace
-                    API.assert{false}
-                    return
                 rescue ForwardAsyncException => e
+                    @terminate = true
                     $stderr.puts e.message
                     $stderr.puts e.backtrace
-                    API.assert{false}
                     return
                 rescue
+                    return_null_response = true
                     $stderr.puts $!.message
                     $stderr.puts $!.backtrace
+                end
+                if return_null_response
                     response_info = ''
                     response = ''
                 end
@@ -402,7 +408,11 @@ module CloudI
 
         def poll_request(timeout, external)
             if @terminate
-                return false
+                if external
+                    return false
+                else
+                    raise TerminateException.new(@timeout_terminate)
+                end
             elsif external and not @initialization_complete
                 send(Erlang.term_to_binary(:polling))
                 @initialization_complete = true
@@ -492,6 +502,9 @@ module CloudI
                     callback(command, name, pattern, request_info, request,
                              request_timeout, priority, trans_id,
                              Erlang.binary_to_term(pid))
+                    if @terminate
+                        return false
+                    end
                 when MESSAGE_RECV_ASYNC, MESSAGE_RETURN_SYNC
                     i += j; j = 4
                     response_info_size = data[i, j].unpack('L')[0]
@@ -615,24 +628,54 @@ module CloudI
             return poll_request(timeout, true)
         end
 
-        def text_key_value_parse(text)
-            result = {}
-            data = text.split(NULL.chr)
-            (0...(data.length)).step(2).each do |i|
-                value = result[data[i]]
-                if value == nil
-                    result[data[i]] = data[i + 1]
-                elsif value.kind_of?(Array)
-                    value << data[i + 1]
-                else
-                    result[data[i]] = [value, data[i + 1]]
-                end
+        def shutdown(reason=nil)
+            if reason.nil?
+                reason = ''
             end
-            return result
+            send(Erlang.term_to_binary([:shutdown, reason]))
         end
 
-        def info_key_value_parse(message_info)
-            return text_key_value_parse(message_info)
+        def self.text_pairs_parse(text)
+            pairs = {}
+            text_segments = text.split(NULL.chr)
+            (0...(text_segments.length)).step(2).each do |i|
+                value = pairs[text_segments[i]]
+                if value == nil
+                    pairs[text_segments[i]] = text_segments[i + 1]
+                elsif value.kind_of?(Array)
+                    value << text_segments[i + 1]
+                else
+                    pairs[text_segments[i]] = [value, text_segments[i + 1]]
+                end
+            end
+            return pairs
+        end
+
+        def self.text_pairs_new(pairs, response)
+            text_stream = StringIO.new
+            pairs.each do |key, values|
+                if values.kind_of?(String)
+                    text_stream.write "#{key}\0#{values}\0"
+                else
+                    values.each do |value|
+                        text_stream.write "#{key}\0#{value}\0"
+                    end
+                end
+            end
+            text = text_stream.string
+            if response and text == ''
+                return "\0"
+            else
+                return text
+            end
+        end
+
+        def self.info_key_value_parse(info)
+            return text_pairs_parse(info)
+        end
+
+        def self.info_key_value_new(pairs, response = true)
+            return text_pairs_new(pairs, response)
         end
 
         def self.assert
@@ -643,7 +686,8 @@ module CloudI
         private :callback
         private :handle_events
         private :poll_request
-        private :text_key_value_parse
+        private_class_method :text_pairs_parse
+        private_class_method :text_pairs_new
         private
 
         def send(data)
