@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2009-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2009-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2009-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_app).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(application).
 
@@ -85,18 +85,16 @@ test() ->
     {error, Reason :: any()}.
 
 start(_, _) ->
-    application:set_env(cloudi_core, mac_address, cloudi_x_uuid:mac_address()),
-    cloudi_x_quickrand:seed(),
+    ok = application:set_env(cloudi_core, mac_address,
+                             cloudi_x_uuid:mac_address()),
+    ok = cloudi_x_quickrand:seed(),
     PathOrData = case application:get_env(cloudi_core, configuration) of
         {ok, C} ->
             C;
         undefined ->
             % default configuration
             % (it is better to not use this, to have everything fail-fast)
-            [{acl, []},
-             {services, []},
-             {nodes, []},
-             {logging, [{file, "cloudi.log"}]}]
+            []
     end,
     case cloudi_core_i_configuration:load(PathOrData) of
         {ok, Config} ->
