@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.5 {@date} {@time}
+%%% @copyright 2011-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_nodes).
@@ -46,7 +46,8 @@
          dead/1,
          nodes/1,
          status/2,
-         logging_redirect_set/1]).
+         logging_redirect_set/1,
+         node_name/1]).
 
 %% gen_server callbacks
 -export([init/1,
@@ -57,28 +58,31 @@
 -include("cloudi_core_i_constants.hrl").
 -include("cloudi_core_i_configuration.hrl").
 
--type nodes_up() ::
+-type nodes_state() ::
     #{node() :=
       {TimeStart :: cloudi_timestamp:native_monotonic(),
+       ConnectNode :: visible | hidden | undefined,
        TimeDisconnect :: undefined | cloudi_timestamp:native_monotonic(),
        Disconnects :: non_neg_integer()}}.
 
 -record(state,
     {
+        node_name :: nonempty_string(),
         nodes_alive = [] :: list(node()),
         nodes_dead :: list(node()),
-        nodes :: list(node()),
-        nodes_up :: nodes_up(),
+        nodes_all :: list(node()),
+        nodes_state :: nodes_state(),
         nodes_down_durations = cloudi_core_i_status:durations_new()
             :: cloudi_core_i_status:durations(node()),
         logging_redirect :: node() | undefined,
         reconnect_interval :: pos_integer(),
         reconnect_timer :: reference(),
-        listen :: visible | all,
+        listen :: cloudi_x_cpg:listen(),
         connect :: visible | hidden,
         discovery :: #config_nodes_discovery{} | undefined,
         cost :: #{node() | default := float()},
-        cost_precision :: 0..253
+        cost_precision :: 0..253,
+        log_reconnect :: cloudi_service_api:loglevel()
     }).
 
 -define(CATCH_EXIT(F),
@@ -88,12 +92,15 @@
 %%% External interface functions
 %%%------------------------------------------------------------------------
 
-start_link(#config{nodes = #config_nodes{listen = Listen,
-                                         connect = Connect,
-                                         timestamp_type = TimestampType}} =
-           Config) ->
-    applications_set(Listen, Connect, TimestampType),
-    gen_server:start_link({local, ?MODULE}, ?MODULE, [Config], []).
+start_link(#config{
+               nodes = #config_nodes{
+                   listen = ListenNew,
+                   connect = Connect,
+                   timestamp_type = TimestampType}} = Config) ->
+    {ok, ListenOld} = application:get_env(cloudi_x_cpg, node_type),
+    ok = applications_set(ListenNew, Connect, TimestampType),
+    gen_server:start_link({local, ?MODULE}, ?MODULE,
+                          [ListenOld, Config], []).
 
 reconfigure(Config, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE, {reconfigure, Config}, Timeout)).
@@ -113,22 +120,40 @@ status(NodesSelection, Timeout) ->
 logging_redirect_set(Node) when is_atom(Node) ->
     gen_server:cast(?MODULE, {logging_redirect_set, Node}).
 
+node_name(Node) ->
+    [[_ | _] = NodeName,
+     _] = cloudi_string:split("@", erlang:atom_to_list(Node)),
+    NodeName.
+
 %%%------------------------------------------------------------------------
 %%% Callback functions from gen_server
 %%%------------------------------------------------------------------------
 
-init([#config{logging = #config_logging{redirect = NodeLogger},
-              nodes = #config_nodes{nodes = Nodes,
-                                    reconnect_start = ReconnectStart,
-                                    reconnect_delay = ReconnectDelay,
-                                    listen = Listen,
-                                    connect = Connect,
-                                    discovery = Discovery,
-                                    cost = Cost,
-                                    cost_precision = CostPrecision}}]) ->
-    monitor_nodes(true, Listen),
+init([ListenOld,
+      #config{
+          logging = #config_logging{
+              redirect = NodeLogger},
+          nodes = #config_nodes{
+              nodes = Nodes,
+              reconnect_start = ReconnectStart,
+              reconnect_delay = ReconnectDelay,
+              listen = ListenNew,
+              connect = Connect,
+              discovery = Discovery,
+              cost = Cost,
+              cost_precision = CostPrecision,
+              log_reconnect = LogReconnect}}]) ->
+    Node = node(),
+    true = cloudi_x_cpg:valid_node(Node),
+    NodeName = node_name(Node),
+
+    % cpg is already running as an application dependency,
+    % so update the listen value
+    ok = monitor_nodes(true, ListenOld),
+    ok = listen_reset(ListenOld, ListenNew, [], NodeName),
+
     NodeLoggerNew = if
-        NodeLogger == node(); NodeLogger =:= undefined ->
+        NodeLogger =:= Node; NodeLogger =:= undefined ->
             undefined;
         true ->
             NodeLogger
@@ -146,89 +171,74 @@ init([#config{logging = #config_logging{redirect = NodeLogger},
         true ->
             ok
     end,
-    NodesUp = #{node() => {erlang:system_info(start_time), undefined, 0}},
-    discovery_start(Discovery),
+    NodesState = #{Node => {erlang:system_info(start_time), undefined,
+                            undefined, 0}},
+    ok = connect_nodes(Nodes, Connect),
+    ok = discovery_start(Discovery),
     ReconnectInterval = ReconnectDelay * 1000,
     ReconnectTimer = erlang:send_after(ReconnectStart * 1000,
                                        self(), reconnect),
-    {ok, #state{nodes_dead = Nodes,
-                nodes = Nodes,
-                nodes_up = NodesUp,
+    {ok, #state{node_name = NodeName,
+                nodes_dead = Nodes,
+                nodes_all = Nodes,
+                nodes_state = NodesState,
                 logging_redirect = NodeLoggerNew,
                 reconnect_interval = ReconnectInterval,
                 reconnect_timer = ReconnectTimer,
-                listen = Listen,
+                listen = ListenNew,
                 connect = Connect,
                 discovery = Discovery,
                 cost = maps:from_list(Cost),
-                cost_precision = CostPrecision}}.
+                cost_precision = CostPrecision,
+                log_reconnect = LogReconnect}}.
 
 handle_call({reconfigure,
-             #config{logging = #config_logging{redirect = NodeLogger},
-                     nodes = #config_nodes{nodes = Nodes,
-                                           reconnect_delay = ReconnectDelay,
-                                           listen = Listen,
-                                           connect = Connect,
-                                           timestamp_type = TimestampType,
-                                           discovery = Discovery,
-                                           cost = Cost,
-                                           cost_precision = CostPrecision}}}, _,
-            #state{nodes_alive = NodesAlive,
-                   nodes_up = NodesUp,
-                   nodes_down_durations = NodesDownDurations,
+             #config{
+                 logging = #config_logging{
+                     redirect = NodeLoggerNew},
+                 services = ServicesNew,
+                 nodes = #config_nodes{
+                     nodes = NodesNew,
+                     reconnect_delay = ReconnectDelayNew,
+                     listen = ListenNew,
+                     connect = ConnectNew,
+                     timestamp_type = TimestampTypeNew,
+                     discovery = DiscoveryNew,
+                     cost = CostNew,
+                     cost_precision = CostPrecisionNew,
+                     log_reconnect = LogReconnectNew}}}, _,
+            #state{node_name = NodeName,
+                   nodes_state = NodesStateOld,
+                   nodes_down_durations = NodesDownDurationsOld,
                    listen = ListenOld,
                    connect = ConnectOld,
                    discovery = DiscoveryOld} = State) ->
-    ConnectedNodes = if
-        Connect =:= visible ->
-            erlang:nodes();
-        Connect =:= hidden ->
-            erlang:nodes(connected)
-    end,
-    NodesNew = lists:usort(Nodes ++ ConnectedNodes),
-    NodesDeadNew = lists:foldl(fun(N, L0) ->
-        case cloudi_lists:delete_checked(N, L0) of
-            false ->
-                % node is alive, but is no longer configured
-                net_kernel:disconnect(N),
-                L0;
-            LN ->
-                LN
-        end
-    end, NodesNew, NodesAlive),
-    NodesAliveNew = lists:filter(fun(N) ->
-        not lists:member(N, NodesDeadNew)
-    end, NodesNew),
-    NodesUpNew = maps:with([node() | NodesNew], NodesUp),
-    NodesDownDurationsNew = cloudi_core_i_status:
-                            durations_copy(NodesNew, NodesDownDurations),
-    ReconnectInterval = ReconnectDelay * 1000,
-    logging_redirect_set(NodeLogger),
-    applications_set(Listen, Connect, TimestampType),
-    if
-        ListenOld /= Listen ->
-            monitor_nodes_switch(ListenOld, Listen),
-            cpg_scopes_reset();
-        true ->
-            ok
-    end,
-    if
-        ConnectOld /= Connect ->
-            discovery_stop(DiscoveryOld),
-            discovery_start(Discovery);
-        true ->
-            discovery_update(DiscoveryOld, Discovery)
-    end,
+    {NodesAliveNew,
+     NodesDeadNew,
+     NodesAllNew,
+     NodesStateNew,
+     NodesDownDurationsNew} = reconfigure_nodes(NodeName,
+                                                NodesNew,
+                                                NodesStateOld,
+                                                NodesDownDurationsOld,
+                                                ListenNew),
+    ReconnectIntervalNew = ReconnectDelayNew * 1000,
+    logging_redirect_set(NodeLoggerNew),
+    ok = applications_set(ListenNew, ConnectNew, TimestampTypeNew),
+    ok = listen_reset(ListenOld, ListenNew, ServicesNew, NodeName),
+    ok = connect_reset(ConnectOld, ConnectNew, DiscoveryOld, DiscoveryNew),
     {reply, ok, State#state{nodes_alive = NodesAliveNew,
                             nodes_dead = NodesDeadNew,
-                            nodes = NodesNew,
-                            nodes_up = NodesUpNew,
+                            nodes_all = NodesAllNew,
+                            nodes_state = NodesStateNew,
                             nodes_down_durations = NodesDownDurationsNew,
-                            reconnect_interval = ReconnectInterval,
-                            connect = Connect,
-                            discovery = Discovery,
-                            cost = maps:from_list(Cost),
-                            cost_precision = CostPrecision}};
+                            reconnect_interval = ReconnectIntervalNew,
+                            listen = ListenNew,
+                            connect = ConnectNew,
+                            discovery = DiscoveryNew,
+                            cost = maps:from_list(CostNew),
+                            cost_precision = CostPrecisionNew,
+                            log_reconnect = LogReconnectNew}};
 
 handle_call(alive, _,
             #state{nodes_alive = NodesAlive} = State) ->
@@ -239,24 +249,24 @@ handle_call(dead, _,
     {reply, {ok, NodesDead}, State};
 
 handle_call(nodes, _,
-            #state{nodes = Nodes} = State) ->
-    {reply, {ok, Nodes}, State};
+            #state{nodes_all = NodesAll} = State) ->
+    {reply, {ok, NodesAll}, State};
 
 handle_call({status, NodesSelection}, _,
-            #state{nodes = Nodes,
-                   nodes_up = NodesUp,
+            #state{nodes_all = NodesAll,
+                   nodes_state = NodesState,
                    nodes_down_durations = NodesDownDurations,
                    cost = Cost,
                    cost_precision = CostPrecision} = State) ->
     TimeNow = cloudi_timestamp:native_monotonic(),
     NodesSelectionNew = if
         NodesSelection == [] ->
-            lists:umerge(Nodes, [node()]);
+            lists:umerge(NodesAll, [node()]);
         true ->
             NodesSelection
     end,
     Reply = nodes_status(NodesSelectionNew, TimeNow,
-                         NodesDownDurations, NodesUp,
+                         NodesDownDurations, NodesState,
                          Cost, CostPrecision),
     {reply, Reply, State};
 
@@ -305,43 +315,53 @@ handle_cast(Request, State) ->
     {stop, cloudi_string:format("Unknown cast \"~w\"", [Request]), State}.
 
 handle_info({nodeup, Node, InfoList},
-            #state{logging_redirect = NodeLogger} = State) ->
-    if
-        Node == NodeLogger ->
-            cloudi_core_i_logger:redirect_update(NodeLogger);
-        true ->
-            ok
+            #state{node_name = NodeName,
+                   logging_redirect = NodeLogger} = State) ->
+    Ignore = cloudi_x_cpg:ignore_node(Node, NodeName),
+    StateNew = if
+        Ignore =:= true ->
+            State;
+        Ignore =:= false ->
+            if
+                Node =:= NodeLogger ->
+                    cloudi_core_i_logger:redirect_update(NodeLogger);
+                true ->
+                    ok
+            end,
+            ?LOG_INFO("nodeup ~p~n ~p", [Node, InfoList]),
+            {node_type, ConnectNode} = lists:keyfind(node_type, 1, InfoList),
+            track_nodeup(Node, ConnectNode, State)
     end,
-    ?LOG_INFO("nodeup ~p~n ~p", [Node, InfoList]),
-    {noreply, track_nodeup(Node, State)};
+    {noreply, StateNew};
 
 handle_info({nodedown, Node, InfoList},
-            #state{logging_redirect = NodeLogger} = State) ->
-    if
-        Node == NodeLogger ->
-            cloudi_core_i_logger:redirect_update(undefined);
-        true ->
-            ok
+            #state{node_name = NodeName,
+                   logging_redirect = NodeLogger} = State) ->
+    Ignore = cloudi_x_cpg:ignore_node(Node, NodeName),
+    StateNew = if
+        Ignore =:= true ->
+            State;
+        Ignore =:= false ->
+            if
+                Node =:= NodeLogger ->
+                    cloudi_core_i_logger:redirect_update(undefined);
+                true ->
+                    ok
+            end,
+            ?LOG_INFO("nodedown ~p~n ~p", [Node, InfoList]),
+            track_nodedown(Node, State)
     end,
-    ?LOG_INFO("nodedown ~p~n ~p", [Node, InfoList]),
-    {noreply, track_nodedown(Node, State)};
+    {noreply, StateNew};
 
 handle_info(reconnect,
             #state{nodes_dead = NodesDead,
+                   nodes_state = NodesState,
                    reconnect_interval = ReconnectInterval,
                    connect = Connect,
-                   discovery = Discovery} = State) ->
+                   discovery = Discovery,
+                   log_reconnect = LogReconnect} = State) ->
     discovery_check(Discovery),
-    if
-        NodesDead /= [] ->
-            ?LOG_INFO("currently dead nodes ~p", [NodesDead]),
-            pforeach(fun(Node) ->
-                % avoid the possibly long synchronous call here
-                connect_node(Connect, Node)
-            end, NodesDead);
-        true ->
-            ok
-    end,
+    ok = reconnect_nodes(NodesDead, NodesState, Connect, LogReconnect),
     ReconnectTimer = erlang:send_after(ReconnectInterval, self(), reconnect),
     {noreply, State#state{reconnect_timer = ReconnectTimer}};
 
@@ -365,6 +385,10 @@ code_change(_, State, _) ->
 %%%------------------------------------------------------------------------
 
 applications_set(Listen, Connect, TimestampType) ->
+    true = (Listen =:= visible) orelse (Listen =:= all),
+    true = (Connect =:= visible) orelse (Connect =:= hidden),
+    true = (TimestampType =:= erlang) orelse
+           (TimestampType =:= os) orelse (TimestampType =:= warp),
     application:set_env(cloudi_x_cpg, node_type, Listen),
     application:set_env(cloudi_x_nodefinder, node_type, Connect),
     application:set_env(cloudi_core, timestamp_type, TimestampType),
@@ -373,55 +397,93 @@ applications_set(Listen, Connect, TimestampType) ->
 monitor_nodes(Flag, Listen) ->
     net_kernel:monitor_nodes(Flag, [{node_type, Listen}, nodedown_reason]).
 
-monitor_nodes_switch(ListenOld, ListenNew) ->
-    % may cause duplicate nodeup/nodedown messages to avoid ignoring events
-    monitor_nodes(true, ListenNew),
-    monitor_nodes(false, ListenOld).
+reconfigure_nodes(NodeName, NodesNew,
+                  NodesStateOld, NodesDownDurationsOld, ListenNew) ->
+    ListenNodes = cloudi_x_cpg:listen_nodes(ListenNew, NodeName),
+    NodesAllNew = lists:usort(ListenNodes ++ NodesNew),
+    NodesDeadNew = reconfigure_nodes_dead(ListenNodes, NodesAllNew),
+    NodesAliveNew = reconfigure_nodes_alive(NodesDeadNew, NodesAllNew),
+    NodesStateNew = maps:with([node() | NodesAllNew], NodesStateOld),
+    NodesDownDurationsNew = cloudi_core_i_status:
+                            durations_copy(NodesAllNew, NodesDownDurationsOld),
+    {NodesAliveNew, NodesDeadNew, NodesAllNew,
+     NodesStateNew, NodesDownDurationsNew}.
 
-track_nodeup(Node,
+reconfigure_nodes_dead([], Nodes) ->
+    Nodes;
+reconfigure_nodes_dead([NodeAlive | NodesAlive], Nodes) ->
+    reconfigure_nodes_dead(NodesAlive, lists:delete(NodeAlive, Nodes)).
+
+reconfigure_nodes_alive([], Nodes) ->
+    Nodes;
+reconfigure_nodes_alive([NodeDead | NodesDead], Nodes) ->
+    reconfigure_nodes_alive(NodesDead, lists:delete(NodeDead, Nodes)).
+
+reconnect_nodes([], _, _, _) ->
+    ok;
+reconnect_nodes(NodesDead, NodesState, Connect, LogReconnect) ->
+    ?LOG(LogReconnect, "currently dead nodes ~p", [NodesDead]),
+    ok = reconnect_node(NodesDead, NodesState, Connect),
+    ok.
+
+reconnect_node([], _, _) ->
+    ok;
+reconnect_node([NodeDead | NodesDead], NodesState, Connect) ->
+    case maps:find(NodeDead, NodesState) of
+        {ok, {_, ConnectNode, _, _}} ->
+            ok = connect_node_async(ConnectNode, NodeDead);
+        error ->
+            ok = connect_node_async(Connect, NodeDead)
+    end,
+    reconnect_node(NodesDead, NodesState, Connect).
+
+track_nodeup(Node, ConnectNode,
              #state{nodes_alive = NodesAlive,
                     nodes_dead = NodesDead,
-                    nodes = Nodes,
-                    nodes_up = NodesUp,
+                    nodes_all = NodesAll,
+                    nodes_state = NodesState,
                     nodes_down_durations = NodesDownDurations} = State) ->
-    TimeUp = cloudi_timestamp:native_monotonic(),
+    TimeConnect = cloudi_timestamp:native_monotonic(),
     NodeL = [Node],
-    {NodesUpNew,
-     NodesDownDurationsNew} = case maps:find(Node, NodesUp) of
-        {ok, {_, undefined, _}} ->
-            {NodesUp, NodesDownDurations}; % duplicate nodeup
-        {ok, {TimeStart, TimeDisconnect, Disconnects}} ->
-            Duration = {TimeDisconnect, TimeUp},
-            {maps:put(Node, {TimeStart, undefined, Disconnects}, NodesUp),
+    {NodesStateNew,
+     NodesDownDurationsNew} = case maps:find(Node, NodesState) of
+        {ok, {_, _, undefined, _}} ->
+            {NodesState, NodesDownDurations}; % duplicate nodeup
+        {ok, {TimeStart, _, TimeDisconnect, Disconnects}} ->
+            NodeState = {TimeStart, ConnectNode, undefined, Disconnects},
+            Duration = {TimeDisconnect, TimeConnect},
+            {maps:put(Node, NodeState, NodesState),
              cloudi_core_i_status:
              durations_store(NodeL, Duration, NodesDownDurations)};
         error ->
-            {maps:put(Node, {TimeUp, undefined, 0}, NodesUp),
+            NodeState = {TimeConnect, ConnectNode, undefined, 0},
+            {maps:put(Node, NodeState, NodesState),
              NodesDownDurations}
     end,
     State#state{nodes_alive = lists:umerge(NodesAlive, NodeL),
                 nodes_dead = lists:delete(Node, NodesDead),
-                nodes = lists:umerge(Nodes, NodeL),
-                nodes_up = NodesUpNew,
+                nodes_all = lists:umerge(NodesAll, NodeL),
+                nodes_state = NodesStateNew,
                 nodes_down_durations = NodesDownDurationsNew}.
 
 track_nodedown(Node,
                #state{nodes_alive = NodesAlive,
                       nodes_dead = NodesDead,
-                      nodes_up = NodesUp} = State) ->
+                      nodes_state = NodesState} = State) ->
     TimeDisconnect = cloudi_timestamp:native_monotonic(),
-    NodesUpNew = case maps:find(Node, NodesUp) of
-        {ok, {TimeStart, undefined, Disconnects}} ->
-            maps:put(Node,
-                     {TimeStart, TimeDisconnect, Disconnects + 1}, NodesUp);
+    NodesStateNew = case maps:find(Node, NodesState) of
+        {ok, {TimeStart, ConnectNode, undefined, Disconnects}} ->
+            NodeState = {TimeStart, ConnectNode,
+                         TimeDisconnect, Disconnects + 1},
+            maps:put(Node, NodeState, NodesState);
         _ ->
-            NodesUp % duplicate nodedown
+            NodesState % duplicate nodedown
     end,
     State#state{nodes_alive = lists:delete(Node, NodesAlive),
                 nodes_dead = lists:umerge(NodesDead, [Node]),
-                nodes_up = NodesUpNew}.
+                nodes_state = NodesStateNew}.
 
-nodes_status(NodesSelection, TimeNow, NodesDownDurations, NodesUp,
+nodes_status(NodesSelection, TimeNow, NodesDownDurations, NodesState,
              Cost, CostPrecision) ->
     TimeDayStart = TimeNow - ?NATIVE_TIME_IN_DAY,
     TimeWeekStart = TimeNow - ?NATIVE_TIME_IN_WEEK,
@@ -429,7 +491,7 @@ nodes_status(NodesSelection, TimeNow, NodesDownDurations, NodesUp,
     TimeYearStart = TimeNow - ?NATIVE_TIME_IN_YEAR,
     CostDefault = maps:get(default, Cost, undefined),
     nodes_status(NodesSelection, [], TimeNow, TimeDayStart, TimeWeekStart,
-                 TimeMonthStart, TimeYearStart, NodesDownDurations, NodesUp,
+                 TimeMonthStart, TimeYearStart, NodesDownDurations, NodesState,
                  Cost, CostDefault, CostPrecision).
 
 nodes_status([], StatusList, _, _, _, _, _, _, _, _, _, _) ->
@@ -437,10 +499,10 @@ nodes_status([], StatusList, _, _, _, _, _, _, _, _, _, _) ->
 nodes_status([Node | NodesSelection], StatusList, TimeNow,
              TimeDayStart, TimeWeekStart,
              TimeMonthStart, TimeYearStart,
-             NodesDownDurations, NodesUp,
+             NodesDownDurations, NodesState,
              Cost, CostDefault, CostPrecision) ->
-    case maps:find(Node, NodesUp) of
-        {ok, {TimeStart, TimeDisconnect, Disconnects}} ->
+    case maps:find(Node, NodesState) of
+        {ok, {TimeStart, ConnectNode, TimeDisconnect, Disconnects}} ->
             LocalNode = Node =:= node(),
             {Disconnected,
              NodesDownDurationsTmp} = if
@@ -567,7 +629,8 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                 LocalNode =:= true ->
                     Status8;
                 LocalNode =:= false ->
-                    [{tracked_disconnects,
+                    [{connection, ConnectNode},
+                     {tracked_disconnects,
                       erlang:integer_to_list(Disconnects)},
                      {disconnected, Disconnected} | Status8]
             end,
@@ -584,7 +647,7 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                          [{Node, StatusN} | StatusList], TimeNow,
                          TimeDayStart, TimeWeekStart,
                          TimeMonthStart, TimeYearStart,
-                         NodesDownDurations, NodesUp,
+                         NodesDownDurations, NodesState,
                          Cost, CostDefault, CostPrecision);
         error ->
             {error, {node_not_found, Node}}
@@ -685,7 +748,7 @@ discovery_start(#config_nodes_discovery{module = Module,
         {ok, _} ->
             ok;
         {error, Reason} ->
-            ?LOG_ERROR("~p:~p error: ~p", [Module, StartF, Reason])
+            ?LOG_ERROR("~p:~tp error: ~tp", [Module, StartF, Reason])
     end,
     ok.
 
@@ -698,7 +761,7 @@ discovery_check(#config_nodes_discovery{module = Module,
         ok ->
             ok;
         {error, Reason} ->
-            ?LOG_ERROR("~p:~p error: ~p", [Module, DiscoverF, Reason])
+            ?LOG_ERROR("~p:~tp error: ~tp", [Module, DiscoverF, Reason])
     end,
     ok.
 
@@ -711,16 +774,16 @@ discovery_stop(#config_nodes_discovery{module = Module,
         ok ->
             ok;
         {error, Reason} ->
-            ?LOG_ERROR("~p:~p error: ~p", [Module, StopF, Reason])
+            ?LOG_ERROR("~p:~tp error: ~tp", [Module, StopF, Reason])
     end,
     ok.
 
 discovery_update(undefined, undefined) ->
     ok;
 discovery_update(undefined, #config_nodes_discovery{} = DiscoveryNew) ->
-    discovery_start(DiscoveryNew);
+    ok = discovery_start(DiscoveryNew);
 discovery_update(#config_nodes_discovery{} = DiscoveryOld, undefined) ->
-    discovery_stop(DiscoveryOld);
+    ok = discovery_stop(DiscoveryOld);
 discovery_update(#config_nodes_discovery{start_f = StartF,
                                          start_a = StartA},
                  #config_nodes_discovery{start_f = StartF,
@@ -728,30 +791,73 @@ discovery_update(#config_nodes_discovery{start_f = StartF,
     ok;
 discovery_update(#config_nodes_discovery{} = DiscoveryOld,
                  #config_nodes_discovery{} = DiscoveryNew) ->
-    discovery_stop(DiscoveryOld),
-    discovery_start(DiscoveryNew).
+    ok = discovery_stop(DiscoveryOld),
+    ok = discovery_start(DiscoveryNew).
 
-cpg_scopes() ->
-    % due to settings in cloudi_core_i_constants.hrl of
-    % SCOPE_CUSTOM_PREFIX and SCOPE_DEFAULT
-    CustomScopes = lists:filter(fun(RegisteredName) ->
-        lists:prefix(?SCOPE_CUSTOM_PREFIX,
-                     erlang:atom_to_list(RegisteredName))
-    end, erlang:registered()),
-    [?SCOPE_DEFAULT | CustomScopes].
+listen_reset(Listen, Listen, _, _) ->
+    ok;
+listen_reset(ListenOld, ListenNew, ServicesNew, NodeName) ->
+    % may cause duplicate nodeup/nodedown messages to avoid ignoring events
+    ok = monitor_nodes(true, ListenNew),
+    HiddenNodesBefore = cloudi_x_cpg:hidden_nodes(NodeName),
+    ok = monitor_nodes(false, ListenOld),
+    if
+        ListenNew =:= all ->
+            visible = ListenOld,
+            ok = listen_reset_all(HiddenNodesBefore);
+        ListenNew =:= visible ->
+            all = ListenOld,
+            HiddenNodesAfter = cloudi_x_cpg:hidden_nodes(NodeName),
+            ok = listen_reset_visible(lists:usort(HiddenNodesBefore ++
+                                                  HiddenNodesAfter))
+    end,
+    ok = cpg_scopes_reset(ServicesNew).
 
-cpg_scopes_reset() ->
-    lists:foreach(fun(Scope) ->
-        cloudi_x_cpg:reset(Scope)
-    end, cpg_scopes()).
+listen_reset_all([]) ->
+    ok;
+listen_reset_all([HiddenNode | HiddenNodes]) ->
+    ?MODULE ! {nodeup, HiddenNode, [{node_type, hidden}]},
+    listen_reset_all(HiddenNodes).
+
+listen_reset_visible([]) ->
+    ok;
+listen_reset_visible(HiddenNodes) ->
+    ?LOG_ERROR("listen should be 'all' for monitoring:~n    ~p",
+               [HiddenNodes]).
+
+connect_reset(Connect, Connect, DiscoveryOld, DiscoveryNew) ->
+    ok = discovery_update(DiscoveryOld, DiscoveryNew);
+connect_reset(_, _, DiscoveryOld, DiscoveryNew) ->
+    ok = discovery_stop(DiscoveryOld),
+    ok = discovery_start(DiscoveryNew).
+
+cpg_scopes([], ScopesSet) ->
+    sets:to_list(ScopesSet);
+cpg_scopes([#config_service_internal{options = Options} | Services],
+           ScopesSet) ->
+    #config_service_options{scope = Scope} = Options,
+    cpg_scopes(Services, sets:add_element(Scope, ScopesSet));
+cpg_scopes([#config_service_external{options = Options} | Services],
+           ScopesSet) ->
+    #config_service_options{scope = Scope} = Options,
+    cpg_scopes(Services, sets:add_element(Scope, ScopesSet)).
+
+cpg_scopes_reset(Services) ->
+    Scopes = cpg_scopes(Services, sets:from_list([?SCOPE_DEFAULT])),
+    cloudi_x_cpg:reset_all(Scopes).
+
+connect_nodes([], _) ->
+    ok;
+connect_nodes([Node | Nodes], Connect) ->
+    ok = connect_node_async(Connect, Node),
+    connect_nodes(Nodes, Connect).
+
+connect_node_async(Connect, Node) ->
+    _ = erlang:spawn_link(fun() -> connect_node(Connect, Node) end),
+    ok.
 
 connect_node(visible, Node) ->
     net_kernel:connect_node(Node);
 connect_node(hidden, Node) ->
     net_kernel:hidden_connect_node(Node).
 
-pforeach(_, []) ->
-    ok;
-pforeach(F, L) ->
-    [erlang:spawn_link(fun() -> F(E) end) || E <- L],
-    ok.
