@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,13 +30,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2015-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_os_process).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([limit_validate/1,
@@ -396,13 +396,11 @@ cgroup_set(OSPid, Values)
     ok = cloudi_x_cgroups:destroy(CGroups),
     Result.
 
--spec cgroup_unset(OSPid :: pos_integer() | undefined,
+-spec cgroup_unset(OSPid :: pos_integer(),
                    Values :: cloudi_service_api:cgroup_external()) ->
     ok |
     {error, any()}.
 
-cgroup_unset(undefined, _) ->
-    ok;
 cgroup_unset(_, undefined) ->
     ok;
 cgroup_unset(OSPid, Values)
@@ -412,9 +410,9 @@ cgroup_unset(OSPid, Values)
     {ok, CGroups} = cloudi_x_cgroups:new(),
     if
         UpdateOrCreate =:= true ->
-            % OSPid should have been sent SIGKILL already and may not exist
+            % move the OSPid back to the root cgroup
             _ = cloudi_x_cgroups:update("", [OSPid], [], CGroups),
-            % may not be the last process to die
+            % delete the cgroup path if no OSPids remain in the cgroup path
             _ = cloudi_x_cgroups:delete_recursive(Name, CGroups),
             ok;
         UpdateOrCreate =:= false ->
