@@ -34,9 +34,10 @@
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 
 %-define(NUMTESTS, 10000).
-%-define(TIMEOUT_MAX, 3600000). % ms (1 hour)
 -define(NUMTESTS, 1).
--define(TIMEOUT_MAX, 1800000). % ms (30 minutes)
+-ifndef(CLOUDI_LONG_TEST_TIMEOUT).
+-define(CLOUDI_LONG_TEST_TIMEOUT, 60). % minutes
+-endif.
 
 % for features specific to Erlang/OTP version 21.x (and later versions)
 -ifdef(OTP_RELEASE).
@@ -77,8 +78,8 @@ cloudi_service_terminate(_Reason, _Timeout, _State) ->
 %%%------------------------------------------------------------------------
 
 all() ->
-    [{group, quorum_timeout},
-     {group, quorum_crash}].
+    test_condition([{group, quorum_timeout},
+                    {group, quorum_crash}]).
 
 groups() ->
     [{quorum_timeout, [],
@@ -92,7 +93,7 @@ groups() ->
 
 suite() ->
     [{enable_builtin_hooks, false},
-     {timetrap, ?TIMEOUT_MAX + 100}].
+     {timetrap, {minutes, ?CLOUDI_LONG_TEST_TIMEOUT}}].
 
 init_per_suite(Config) ->
     ok = cloudi_x_reltool_util:
@@ -306,6 +307,14 @@ validate_quorum_crash(QuorumType, UseResponseInfo,
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+test_condition(L) ->
+    if
+        ?CLOUDI_LONG_TEST_TIMEOUT > 0 ->
+            L;
+        ?CLOUDI_LONG_TEST_TIMEOUT =:= 0 ->
+            {skip, long_tests_disabled}
+    end.
+
 services_add(I, Configuration) ->
     services_add(I, [], Configuration).
 
