@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2013-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2013-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_tcp).
@@ -42,7 +42,6 @@
 
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
-         cloudi_service_handle_request/11,
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
@@ -312,11 +311,6 @@ cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
             {stop, Error, #state{}}
     end.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
-                              _Timeout, _Priority, _TransId, _Pid,
-                              State, _Dispatcher) ->
-    {reply, <<>>, State}.
-
 cloudi_service_handle_info({inet_async, Listener, Acceptor, {ok, Socket}},
                            #state{listener = Listener,
                                   acceptor = Acceptor,
@@ -447,7 +441,7 @@ socket_loop_init_set(true, Scope,
                            DestinationSetRequestInfo, <<"SET">>,
                            TimeoutSync, Self) of
         {ok, ResponseInfo, _Response} ->
-            KeyValues = cloudi_request_info:key_value_parse(ResponseInfo),
+            KeyValues = cloudi_response_info:key_value_parse(ResponseInfo),
             NewDestination = case cloudi_key_value:find(<<"service_name">>,
                                                         KeyValues) of
                 error ->
@@ -546,7 +540,7 @@ socket_loop_init(Scope, DestinationSet, DestinationConnect,
         {init, ok} ->
             NewState = socket_loop_init_connect(DestinationConnect,
                 socket_loop_init_set(DestinationSet, Scope, State)),
-            socket_debug_log(DebugLevel, "socket ~p opened", [Socket]),
+            ?LOG(DebugLevel, "socket ~p opened", [Socket]),
             ok = inet:setopts(Socket, [{active, once}]),
             socket_loop(NewState);
         {init, Error} ->
@@ -567,8 +561,7 @@ socket_loop_out_request({Type, Name, Pattern, RequestInfo, Request,
          Timeout, Priority, TransId, Source},
     socket_terminate_check(Socket, RequestInfo),
     socket_send(Request, State),
-    socket_debug_log(DebugLevel,
-                    "socket out request ~p", [Request]),
+    ?LOG(DebugLevel, "socket out request ~p", [Request]),
     State#state_socket{
         response_pending = true,
         response_timer = ResponseTimer,
@@ -588,8 +581,7 @@ socket_loop_out_request({Type, Name, Pattern, RequestInfo, RequestProtocol,
          Timeout, Priority, TransId, Source},
     socket_terminate_check(Socket, RequestInfo),
     socket_send(Request, State),
-    socket_debug_log(DebugLevel,
-                    "socket out request ~p", [Request]),
+    ?LOG(DebugLevel, "socket out request ~p", [Request]),
     State#state_socket{
         response_lookup = maps:put(ID, {T, ResponseTimer}, ResponseLookup)};
 socket_loop_out_request({_, _, _, _, Request,
@@ -621,8 +613,7 @@ socket_loop_in_request(Request, ResponseF,
                            request_info = RequestInfo,
                            lock = Lock,
                            debug_level = DebugLevel} = State) ->
-    socket_debug_log(DebugLevel,
-                     "socket in request ~p", [Request]),
+    ?LOG(DebugLevel, "socket in request ~p", [Request]),
     case send_sync_minimal(Dispatcher, Destination, RequestInfo, Request,
                            TimeoutSync, Lock, self()) of
         {ok, ResponseInfo, Response} ->
@@ -634,15 +625,13 @@ socket_loop_in_request(Request, ResponseF,
                     ResponseF(Response)
             end,
             socket_send(NewResponse, State),
-            socket_debug_log(DebugLevel,
-                             "socket out response ~p", [NewResponse]),
+            ?LOG(DebugLevel, "socket out response ~p", [NewResponse]),
             State;
         {error, timeout} ->
             socket_send(<<>>, State),
             if
                 Lock =:= undefined ->
-                    socket_debug_log(DebugLevel,
-                                     "socket out response ~p", [<<>>]),
+                    ?LOG(DebugLevel, "socket out response ~p", [<<>>]),
                     State;
                 true ->
                     socket_loop_terminate(timeout, State)
@@ -653,8 +642,7 @@ socket_loop_in_response({SendType, Name, Pattern, _, _,
                          _, _, TransId, Source},
                         ResponseTimer, ResponseInfo, Response,
                         DebugLevel) ->
-    socket_debug_log(DebugLevel,
-                     "socket in response ~p", [Response]),
+    ?LOG(DebugLevel, "socket in response ~p", [Response]),
     Timeout = case erlang:cancel_timer(ResponseTimer) of
         false ->
             0;
@@ -788,7 +776,7 @@ socket_loop_terminate(Reason,
     end,
     if
         Reason =:= normal ->
-            socket_debug_log(DebugLevel, "socket ~p closed", [Socket]);
+            ?LOG(DebugLevel, "socket ~p closed", [Socket]);
         true ->
             ?LOG_ERROR("socket ~p error: ~p", [Socket, Reason])
     end,
@@ -798,7 +786,7 @@ socket_loop_terminate(Reason,
     erlang:exit(Reason).
 
 socket_terminate_check(Socket, Info) ->
-    KeyValues = cloudi_request_info:key_value_parse(Info),
+    KeyValues = cloudi_response_info:key_value_parse(Info),
     case cloudi_key_value:find(<<"connection">>, KeyValues) of
         {ok, <<"close">>} ->
             self() ! {tcp_closed, Socket},
@@ -817,21 +805,6 @@ socket_send(Outgoing,
             socket_loop_terminate(normal, State)
     end.
 
-socket_debug_log(off, _, _) ->
-    ok;
-socket_debug_log(trace, Message, Args) ->
-    ?LOG_TRACE(Message, Args);
-socket_debug_log(debug, Message, Args) ->
-    ?LOG_DEBUG(Message, Args);
-socket_debug_log(info, Message, Args) ->
-    ?LOG_INFO(Message, Args);
-socket_debug_log(warn, Message, Args) ->
-    ?LOG_WARN(Message, Args);
-socket_debug_log(error, Message, Args) ->
-    ?LOG_ERROR(Message, Args);
-socket_debug_log(fatal, Message, Args) ->
-    ?LOG_FATAL(Message, Args).
-
 socket_process_queue(#state_socket{recv_timeouts = RecvTimeouts,
                                    queued = Queue} = State) ->
     case cloudi_x_pqueue4:out(Queue) of
