@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,13 +30,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2013-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(reltool_util).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -export([application_env/1,
          application_start/1,
@@ -64,6 +64,7 @@
          module_loaded/1,
          module_unload/1,
          module_reload/1,
+         modules_reload/1,
          is_module_loaded/1,
          is_module_loaded/2,
          module_purged/1,
@@ -664,6 +665,27 @@ module_reload(Module)
             Error
     end.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Reload a list of modules.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec modules_reload(Modules :: nonempty_list(atom())) ->
+    ok |
+    {error, any()}.
+
+modules_reload([Module | _] = Modules)
+    when is_atom(Module) ->
+    ok = modules_map(Modules, purge),
+    case code:atomic_load(Modules) of
+        ok ->
+            ok = modules_map(Modules, soft_purge),
+            ok;
+        {error, _} = Error ->
+            Error
+    end.
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Wait to check if a module is loaded.===
@@ -1555,7 +1577,7 @@ script_remove_instructions([{apply, {c, erlangrc, _}} | L],
 
 ensure_all_loaded([]) ->
     ok;
-ensure_all_loaded([Module | Modules]) ->
+ensure_all_loaded([Module | Modules] = L) ->
     case is_module_loaded_check(Module) of
         true ->
             Loaded = lists:all(fun(M) ->
@@ -1573,7 +1595,7 @@ ensure_all_loaded([Module | Modules]) ->
             end, Modules),
             if
                 NotLoaded ->
-                    load_all_modules([Module | Modules]);
+                    code:atomic_load(L);
                 true ->
                     {error, modules_partially_loaded}
             end
@@ -1677,15 +1699,11 @@ modules_filter([{Behaviour, Name} | Options], Modules, Loaded)
 modules_filter([_ | _], _, _) ->
     erlang:exit(badarg).
 
-load_all_modules([]) ->
+modules_map([], _) ->
     ok;
-load_all_modules([Module | Modules]) ->
-    case code:load_file(Module) of
-        {module, Module} ->
-            load_all_modules(Modules);
-        {error, Reason} ->
-            {error, {Reason, Module}}
-    end.
+modules_map([Module | Modules], Function) ->
+    _ = code:Function(Module),
+    modules_map(Modules, Function).
 
 load_all_paths([], _) ->
     ok;
