@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -52,16 +52,20 @@ request_ requestType name pattern _ request timeout _ transId pid state api =
                 "<http_test><error>no value specified</error></http_test>"
             Just (i) ->
                 "<http_test><value>" ++ (show i) ++ "</value></http_test>"
+        httpResponseHeaders = Map.insert
+            (Char8.pack "content-type")
+            [Char8.pack "text/xml; charset=utf-8"] Map.empty
+        responseInfo = CloudI.infoKeyValueNew httpResponseHeaders Nothing
     in do
     CloudI.return_ api
-        requestType name pattern ByteString.empty response timeout transId pid
+        requestType name pattern responseInfo response timeout transId pid
     return $ CloudI.Null (state, api)
 
 task :: Int -> IO ()
 task threadIndex = do
     let prerr = SysIO.hPutStrLn SysIO.stderr
         prout = putStrLn
-    apiValue <- CloudI.api threadIndex ()
+    apiValue <- CloudI.api threadIndex () Nothing
     case apiValue of
         Left err ->
             prerr err
