@@ -1,41 +1,27 @@
 #-*-Mode:perl;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 # ex: set ft=perl fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
-# BSD LICENSE
-# 
-# Copyright (c) 2014, Michael Truog <mjtruog at gmail dot com>
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-# 
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in
-#       the documentation and/or other materials provided with the
-#       distribution.
-#     * All advertising materials mentioning features or use of this
-#       software must display the following acknowledgment:
-#         This product includes software developed by Michael Truog
-#     * The name of the author may not be used to endorse or promote
-#       products derived from this software without specific prior
-#       written permission
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-# DAMAGE.
+# MIT License
+#
+# Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the "Software"),
+# to deal in the Software without restriction, including without limitation
+# the rights to use, copy, modify, merge, publish, distribute, sublicense,
+# and/or sell copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
 #
 
 package MessagingTask;
@@ -90,7 +76,7 @@ sub run
         if ($self->{thread_index} == 0)
         {
             $self->{api}->send_async(
-                $self->{api}->prefix() . 'sequence1', 'start');
+                $self->{api}->prefix() . 'sequence1', '1');
         }
         my $result = $self->{api}->poll();
         assert($result == 0);
@@ -119,138 +105,135 @@ sub assert
 sub _sequence1_abcd
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/c/d'));
     assert($request eq 'test1');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_abc_
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/c/*'));
     assert($request eq 'test2' || $request eq 'test3');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_ab_d
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/*/d'));
     assert($request eq 'test4' || $request eq 'test5');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_a_cd
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/*/c/d'));
     assert($request eq 'test6' || $request eq 'test7');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1__bcd
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*/b/c/d'));
     assert($request eq 'test8' || $request eq 'test9');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_ab__
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/*'));
     assert($request eq 'test10');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_a__d
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/*/d'));
     assert($request eq 'test11');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1___cd
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*/c/d'));
     assert($request eq 'test12');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_a___
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/*'));
     assert($request eq 'test13');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1____d
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*/d'));
     assert($request eq 'test14');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1_____
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*'));
     assert($request eq 'test15');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request, $timeout, $trans_id, $pid);
 }
 
 sub _sequence1
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     my @end = $self->{api}->recv_async(1000);
     while ($end[1] eq 'end')
     {
         @end = $self->{api}->recv_async(1000);
     }
-    print "messaging sequence1 start perl\n";
-    assert($request eq 'start');
-    # n.b., depends on cloudi_constants.hrl having
-    # SERVICE_NAME_PATTERN_MATCHING defined
+    print "messaging sequence1 start perl ($request)\n";
     my $test1_id = $self->{api}->send_async(
         $self->{api}->prefix() . 'a/b/c/d',  'test1');
     my $test2_id = $self->{api}->send_async(
@@ -281,7 +264,7 @@ sub _sequence1
         $self->{api}->prefix() . 'azbzc/d',  'test14');
     my $test15_id = $self->{api}->send_async(
         $self->{api}->prefix() . 'azbzczd',  'test15');
-    # n.b., depends on cloudi_constants.hrl having
+    # n.b., depends on cloudi_core_i_constants.hrl having
     # RECV_ASYNC_STRATEGY == recv_async_select_oldest
     my $tmp;
     $self->{api}->recv_async(undef, $test1_id, 0);
@@ -374,92 +357,91 @@ sub _sequence1
     ($tmp, $test15_check, $test15_id_check) = $self->{api}->recv_async();
     assert($test15_check eq 'test15');
     assert($test15_id_check eq $test15_id);
-    print "messaging sequence1 end perl\n";
+    print "messaging sequence1 end perl ($request)\n";
     # start sequence2
-    $self->{api}->send_async($self->{api}->prefix() . 'sequence2', 'start');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->send_async($self->{api}->prefix() . 'sequence2', $request);
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', 'end', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e1
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '1', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e2
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '2', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e3
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '3', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e4
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '4', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e5
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '5', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e6
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '6', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e7
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '7', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2_e8
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', '8', $timeout, $trans_id, $pid);
 }
 
 sub _sequence2
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    print "messaging sequence2 start perl\n";
-    assert($request eq 'start');
+    print "messaging sequence2 start perl ($request)\n";
     while (1)
     {
         # the sending process is excluded from the services that receive
@@ -500,24 +482,24 @@ sub _sequence2
             assert($null[2] eq "\0" x 16);
         }
     }
-    print "messaging sequence2 end perl\n";
+    print "messaging sequence2 end perl ($request)\n";
     # start sequence3
-    $self->{api}->send_async($self->{api}->prefix() . 'sequence3', 'start');
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->send_async($self->{api}->prefix() . 'sequence3', $request);
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', 'end', $timeout, $trans_id, $pid);
 }
 
 sub _sequence3_f1
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request_i,
+    my ($request_type, $name, $pattern, $request_info, $request_i,
         $timeout, $priority, $trans_id, $pid) = @_;
     if ($request_i == 4)
     {
         return 'done';
     }
     my $request_new = $request_i + 2; # two steps forward
-    $self->{api}->forward_($command, $self->{api}->prefix() . 'f2',
+    $self->{api}->forward_($request_type, $self->{api}->prefix() . 'f2',
                            $request_info, $request_new,
                            $timeout, $priority, $trans_id, $pid);
 }
@@ -525,10 +507,10 @@ sub _sequence3_f1
 sub _sequence3_f2
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request_i,
+    my ($request_type, $name, $pattern, $request_info, $request_i,
         $timeout, $priority, $trans_id, $pid) = @_;
     my $request_new = $request_i - 1; # one step back
-    $self->{api}->forward_($command, $self->{api}->prefix() . 'f1',
+    $self->{api}->forward_($request_type, $self->{api}->prefix() . 'f1',
                            $request_info, $request_new,
                            $timeout, $priority, $trans_id, $pid);
 }
@@ -536,19 +518,18 @@ sub _sequence3_f2
 sub _sequence3_g1
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    $self->{api}->return_($command, $name, $pattern,
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', $request . 'suffix', $timeout, $trans_id, $pid);
 }
 
 sub _sequence3
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    print "messaging sequence3 start perl\n";
-    assert($request eq 'start');
+    print "messaging sequence3 start perl ($request)\n";
     my $test1_id = $self->{api}->send_async(
         $self->{api}->prefix() . 'f1',  '0');
     my $tmp;
@@ -563,16 +544,21 @@ sub _sequence3
     ($tmp, $test2_check, $test2_id_check) = $self->{api}->send_sync(
         $self->{api}->prefix() . 'g1',  'prefix_');
     assert($test2_check eq 'prefix_suffix');
-    print "messaging sequence3 end perl\n";
+    print "messaging sequence3 end perl ($request)\n";
     # loop to find any infrequent problems, restart sequence1
-    $self->{api}->send_async($self->{api}->prefix() . 'sequence1', 'start');
-    $self->{api}->return_($command, $name, $pattern,
+    my $iteration = $request + 1;
+    if ($iteration == 9007199254740992)
+    {
+        $iteration = 0;
+    }
+    $self->{api}->send_async($self->{api}->prefix() . 'sequence1', $iteration);
+    $self->{api}->return_($request_type, $name, $pattern,
                           '', 'end', $timeout, $trans_id, $pid);
 }
 
 {
     assert($use_threads);
-    my $thread_count = CloudI::API->thread_count();
+    my $thread_count = CloudI::API::thread_count();
     my @threads = ();
     for my $i (0 .. ($thread_count - 1))
     {
